@@ -2,11 +2,11 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Oct  2, 1996
-;; Revised: Aug 24, 1998
+;; Revised: Aug 30, 1999
 
 ;;; Code:
 
-(defconst mew-attach-version "mew-attach.el version 0.17")
+(defconst mew-attach-version "mew-attach.el version 0.18")
 
 (require 'mew)
 
@@ -16,6 +16,7 @@
 (if mew-draft-attach-map
     ()
   (setq mew-draft-attach-map (make-keymap))
+  (define-key mew-draft-attach-map "\C-m" 'mew-attach-newline)
   (define-key mew-draft-attach-map "\C-f" 'mew-attach-forward)
   (define-key mew-draft-attach-map "\C-b" 'mew-attach-backforward)
   (define-key mew-draft-attach-map "\C-n" 'mew-attach-next)
@@ -24,6 +25,7 @@
   (define-key mew-draft-attach-map "c"    'mew-attach-copy)
   (define-key mew-draft-attach-map "C"    'mew-attach-charset)
   (define-key mew-draft-attach-map "d"    'mew-attach-delete)
+  (define-key mew-draft-attach-map "y"    'mew-attach-link-message)
   (define-key mew-draft-attach-map "D"    'mew-attach-description)
   (define-key mew-draft-attach-map "P"    'mew-attach-disposition)
   (define-key mew-draft-attach-map "e"    'mew-attach-external-body)
@@ -39,18 +41,8 @@
   (define-key mew-draft-attach-map "E"    'mew-attach-pgp-enc)
   (define-key mew-draft-attach-map "p"    'mew-attach-pgp-public-key)
   (define-key mew-draft-attach-map "U"    'mew-attach-undo)
-  (define-key mew-draft-attach-map "\C-y" 'mew-attach-dummy)
   (if mew-use-overlay-keymap
       (mew-draft-share-keymap 'mew-draft-attach-map))
-  (let ((begin ?\0) (end ?\177))
-    (while (<= begin end)
-      (if (and (/= begin 27)
-               (/= begin ?\C-u)
-               (/= begin ?\C-g)
-	       (null (lookup-key mew-draft-attach-map (char-to-string begin))))
-	  (define-key mew-draft-attach-map 
-	    (char-to-string begin) 'mew-attach-dummy))
-      (setq begin (1+ begin))))
   )
 
 (if mew-xemacs-p
@@ -76,33 +68,30 @@
   "Prepare an attachment region in the bottom of the draft.
 To compose a multipart message, you should execute this command first."
   (interactive)
-  (mew-draft-before-composition-only
-   (let* ((mimedir (mew-expand-folder (mew-draft-to-mime (buffer-name))))
-	  (cp (expand-file-name mew-draft-coverpage mimedir))
-	  (dir (file-name-nondirectory mimedir)))
-     (if (mew-attach-p)
-	 (message "Attachments already exist.")
-       (mew-attach-set)
-       (if (not (file-exists-p mimedir)) (mew-make-directory mimedir))
-       (if (not (file-exists-p cp)) 
-	   (write-region 1 1 cp 'append 'no-msg));; touch
-       (if (not mew-encode-syntax)
-	   (setq mew-encode-syntax (mew-encode-syntax-initial dir)))
-       (mew-encode-syntax-print mew-encode-syntax)
-       (mew-attach-goto-number 'here '(2))
-       (if (and (not no-scroll)
-		(not (pos-visible-in-window-p
-		      (point-max)
-		      (get-buffer-window (current-buffer)))))
-	   (set-window-start (get-buffer-window (current-buffer))
-			     (mew-attach-begin)))
-       (setq buffer-undo-list nil)))))
-
-(defun mew-attach-nums ()
-  (let ((strnum (mew-syntax-number)))
-    (if strnum
-	(mapcar (function string-to-int) (mew-split strnum ?.))
-      nil)))
+  (if (not (mew-header-p))
+      (message "Attachments region can be prepared before makeing a message only")
+    (if (mew-attach-p)
+	(message "Attachemnts region have been already prepared")
+      (let* ((attachdir (mew-attachdir))
+	     (cp (expand-file-name mew-draft-coverpage attachdir))
+	     (dir (file-name-nondirectory attachdir)))
+	(if (mew-attach-p)
+	    (message "Attachments already exist.")
+	  (mew-attach-set)
+	  (if (not (file-exists-p attachdir)) (mew-make-directory attachdir))
+	  (if (not (file-exists-p cp)) 
+	      (write-region 1 1 cp 'append 'no-msg));; touch
+	  (if (not mew-encode-syntax)
+	      (setq mew-encode-syntax (mew-encode-syntax-initial dir)))
+	  (mew-encode-syntax-print mew-encode-syntax)
+	  (mew-attach-goto-number 'here '(2))
+	  (if (and (not no-scroll)
+		   (not (pos-visible-in-window-p
+			 (point-max)
+			 (get-buffer-window (current-buffer)))))
+	      (set-window-start (get-buffer-window (current-buffer))
+				(mew-attach-begin)))
+	  (setq buffer-undo-list nil))))))
 
 (defun mew-attach-expand-path (syntax nums)
   "ignore the first dir"
@@ -110,8 +99,7 @@ To compose a multipart message, you should execute this command first."
     (while (cdr nums)
       (setq syntax (aref syntax (+ mew-syntax-magic (1- (car nums)))))
       (setq path (concat path (mew-syntax-get-file syntax)))
-      (setq nums (cdr nums))
-      )
+      (setq nums (cdr nums)))
     path))
 
 (defun mew-attach-line ()
@@ -122,19 +110,27 @@ attachments region.
   (let (ret max)
     (save-excursion
       (beginning-of-line)
-      (setq ret (count-lines (mew-attach-begin) (point)))
+      (setq ret (count-lines (1+ (mew-attach-begin)) (point)))
       (goto-char (point-max))
       (beginning-of-line)
-      (setq max (count-lines (mew-attach-begin) (point)))
+      (setq max (count-lines (1+ (mew-attach-begin)) (point)))
       (if (or (= ret max) (= ret (1- max))) -1 ret))))
 
+
+(defun mew-attach-line-lastp ()
+  (let ((here (point)))
+    (save-excursion
+      (goto-char (point-max))
+      (beginning-of-line)
+      (equal (count-lines here (point)) 2))))
+
 (defun mew-attach-move-onto-file ()
   (end-of-line)
-  (re-search-backward " [^ ]")
+  (re-search-backward " [^ ]" nil t)
   (forward-char))
 
 (defun mew-attach-move-onto-body ()
-  (goto-char (mew-attach-begin))
+  (goto-char (1+ (mew-attach-begin)))
   (forward-char -1))
 
 (defun mew-attach-directory-p ()
@@ -198,33 +194,27 @@ attachments region.
     (setq nums (nreverse nums))
     (cond
      ((equal direction 'next)
-      (setq nums (cons (1+ (car nums)) (cdr nums)))
-      )
+      (setq nums (cons (1+ (car nums)) (cdr nums))))
      ((equal direction 'prev)
-      (setq nums (cons (1- (car nums)) (cdr nums)))
-      )
+      (setq nums (cons (1- (car nums)) (cdr nums))))
      ((equal direction 'up)
-      (setq nums (cdr nums))
-      )
+      (setq nums (cdr nums)))
      ((equal direction 'down)
-      (setq nums (cons 1 nums))
-      )
-     (t ())
-     )
+      (setq nums (cons 1 nums)))
+     (t ()))
     (if (null nums)
 	(progn
-	  (goto-char (mew-attach-begin))
-	  (re-search-forward "^..... ")
+	  (goto-char (1- (mew-attach-begin)))
+	  (re-search-forward "^..... " nil t)
 	  (mew-attach-move-onto-file))
       (setq numreg (int-to-string (car nums)))
       (setq nums (cdr nums))
       (while nums
 	(setq numreg (concat (int-to-string (car nums)) "." numreg))
-	(setq nums (cdr nums))
-	)
+	(setq nums (cdr nums)))
       (setq numreg (regexp-quote numreg))
-      (goto-char (mew-attach-begin))
-      (re-search-forward (concat "^....." numreg " "))
+      (goto-char (1+ (mew-attach-begin)))
+      (re-search-forward (concat "^....." numreg " ") nil t)
       (mew-attach-move-onto-file))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -236,6 +226,14 @@ attachments region.
 ;; cursor commands
 ;;
 
+(defun mew-attach-newline ()
+  "Insert RET before the attachments region."
+  (interactive)
+  (if (equal (point) (1+ (mew-attach-begin)))
+      (progn
+	(insert "\n")
+	(mew-draft-attach-keymap))))
+
 (defun mew-attach-forward ()
   "Go to the first subdirectory in attachments."
   (interactive) 
@@ -243,41 +241,47 @@ attachments region.
       (mew-attach-goto-number 'here nil)
     (mew-attach-move-onto-file)
     (if (mew-attach-directory-p)
-	(mew-attach-goto-number 'down (mew-attach-nums))
-      (mew-attach-goto-number 'here (mew-attach-nums)))))
+	(mew-attach-goto-number 'down (mew-syntax-nums))
+      (mew-attach-goto-number 'here (mew-syntax-nums)))))
 
 (defun mew-attach-backforward ()
   "Go to the parent directory in attachments."
   (interactive)
   (if (mew-attach-line0-1)
       (mew-attach-move-onto-body)
-    (let ((nums (mew-attach-nums)))
+    (let ((nums (mew-syntax-nums)))
       (if (= (length nums) 1)
 	  (mew-attach-goto-number 'here nil)
-	(mew-attach-goto-number 'up (mew-attach-nums))))))
+	(mew-attach-goto-number 'up (mew-syntax-nums))))))
 
 (defun mew-attach-previous ()
   "Go to the previous file in the current directory in attachments."
   (interactive)
   (if (mew-attach-line01-1)
       (mew-attach-move-onto-body)
-  (let* ((nums (mew-attach-nums))
-	 (last (nth (1- (length nums)) nums)))
-    (if (= last 1)
-	(mew-attach-goto-number 'here (mew-attach-nums))
-      (mew-attach-goto-number 'prev (mew-attach-nums))))))
+    (if mew-attach-move-by-line
+	(progn
+	  (forward-line -1)
+	  (mew-attach-move-onto-file))
+      (let* ((nums (mew-syntax-nums))
+	     (last (nth (1- (length nums)) nums)))
+	(if (= last 1)
+	    (mew-attach-goto-number 'here (mew-syntax-nums))
+	  (mew-attach-goto-number 'prev (mew-syntax-nums)))))))
 
 (defun mew-attach-next ()
   "Go to the next file in the current directory in attachments."
   (interactive)
-  (cond
-   ((mew-attach-line0-1)
-    (mew-attach-goto-number 'here nil))
-   ((mew-attach-line1-dot)
-    (mew-attach-goto-number 'here (mew-attach-nums)))
-   (t
-    (mew-attach-goto-number 'next (mew-attach-nums)))
-   ))
+  (if (mew-attach-line0-1)
+      (mew-attach-goto-number 'here nil)
+    (if mew-attach-move-by-line
+	(if (mew-attach-line-lastp)
+	    ()
+	  (forward-line)
+	  (mew-attach-move-onto-file))
+      (if (mew-attach-line1-dot)
+	  (mew-attach-goto-number 'here (mew-syntax-nums))
+	(mew-attach-goto-number 'next (mew-syntax-nums))))))
 
 ;;
 ;; delete commands
@@ -291,8 +295,7 @@ attachments region.
     (if (equal (mew-attach-line) 1)
 	(if (y-or-n-p "Delete entire attachments? ")
 	    (progn
-	      (mew-delete-directory-recursively
-	       (mew-expand-folder (mew-draft-to-mime (buffer-name))))
+	      (mew-delete-directory-recursively (mew-attachdir))
 	      (setq mew-encode-syntax nil)
 	      (mew-attach-clear)
 	      (if mew-icon-p
@@ -300,18 +303,18 @@ attachments region.
 				 (cons (current-buffer) mew-draft-toolbar)))
 	      (message "attachments were deleted"))
 	  (message "attachments were not deleted"))
-      (let* ((nums (mew-attach-nums))
+      (let* ((nums (mew-syntax-nums))
 	     (subdir (mew-attach-expand-path mew-encode-syntax nums))
-	     (mimedir (mew-expand-folder (mew-draft-to-mime (buffer-name))))
+	     (attachdir (mew-attachdir))
 	     (syntax (mew-syntax-get-entry mew-encode-syntax nums))
 	     (name (mew-syntax-get-file syntax))
 	     (ename (if (equal subdir "") name (concat subdir name)))
-	     (fullname (expand-file-name ename mimedir))
+	     (fullname (expand-file-name ename attachdir))
 	     (dirp (string-match (concat mew-path-separator "$") name))
 	     (msg (if dirp
 		      "Delete %s with contained files? "
 		    "Delete %s? ")))
-	;; mimedir / {subdir/} name
+	;; attachdir / {subdir/} name
 	(if (not (y-or-n-p (format msg ename)))
 	    ()
 	  (message "Deleting %s ... " ename)
@@ -329,20 +332,20 @@ attachments region.
 ;;
 
 (defun mew-attach-multipart ()
-  "Create a subdirectory(i.e. multipart) on \".\" in attachments."
+  "Create a subdirectory(i.e. multipart) on '.' in attachments."
   (interactive)
   (if (not (mew-attach-not-line012-1))
       (message "Can't create a sub-multipart here.")
-    (let* ((nums (mew-attach-nums))
+    (let* ((nums (mew-syntax-nums))
 	   (subdir (mew-attach-expand-path mew-encode-syntax nums))
 	   (insert-default-directory nil)
-	   (mimedir (mew-expand-folder (mew-draft-to-mime (buffer-name))))
+	   (attachdir (mew-attachdir))
 	   dir dirpath)
-      ;; mimedir / {subdir/} dir
+      ;; attachdir / {subdir/} dir
       (if (not (equal subdir "")) 
-	  (setq mimedir (expand-file-name subdir mimedir)))
-      ;; mimedir / dir
-      (setq dirpath (mew-random-filename mimedir))
+	  (setq attachdir (expand-file-name subdir attachdir)))
+      ;; attachdir / dir
+      (setq dirpath (mew-random-filename attachdir))
       (if (file-exists-p dirpath)
 	  (message "Could not make a directory, sorry")
 	(setq dir (file-name-nondirectory dirpath))
@@ -354,29 +357,31 @@ attachments region.
 	       (mew-encode-syntax-multi dir mew-type-mlm)))
 	(mew-encode-syntax-print mew-encode-syntax)))))
 
-(defun mew-attach-duplicate (func action setmode &optional from to)
+(defun mew-attach-duplicate (func action setmode &optional from to default)
   (if (not (mew-attach-not-line012-1))
       (message "Can't %s here." action)
-    (let* ((nums (mew-attach-nums))
+    (let* ((nums (mew-syntax-nums))
 	   (subdir (mew-attach-expand-path mew-encode-syntax nums))
-	   (mimedir (mew-expand-folder (mew-draft-to-mime (buffer-name))))
+	   (attachdir (mew-attachdir))
 	   (frompath from)
 	   (efile (and to (concat subdir to)))
-	   (tofile to))
+	   (tofile to)
+	   FILE)
       (while (or (null frompath)
 		 (file-directory-p (file-chase-links frompath)))
-	(setq frompath (mew-input-file-name
-			(concat (capitalize action) " from : "))))
+	(setq frompath (mew-draft-input-file-name
+			(concat (capitalize action) " from : ") default)))
       (while (or (null efile)
-		 (file-exists-p (expand-file-name efile mimedir)))
+		 (file-exists-p (expand-file-name efile attachdir)))
 	(setq tofile (mew-input-string 
 		      (concat (capitalize action) " to %s(%s): ")
 		      subdir
 		      (file-name-nondirectory frompath)))
 	(setq efile (concat subdir tofile)))
-      (funcall func frompath (expand-file-name efile mimedir))
-      (if setmode (set-file-modes (expand-file-name efile mimedir)
-				  mew-file-mode))
+      (setq FILE (expand-file-name efile attachdir))
+      (funcall func frompath FILE)
+      (if (and setmode (/= mew-file-mode (mew-file-get-mode FILE)))
+	  (set-file-modes FILE mew-file-mode))
       (setq mew-encode-syntax
 	    (mew-syntax-insert-entry
 	     mew-encode-syntax 
@@ -385,32 +390,44 @@ attachments region.
       (mew-encode-syntax-print mew-encode-syntax))))
 
 (defun mew-attach-link (&optional from to)
-  "Link a file with a symbolic link on \".\" in attachments.
+  "Link a file with a symbolic link on '.' in attachments.
 FROM must be absolute path. TO must be a file name."
   (interactive)
   (mew-attach-duplicate (function mew-symbolic-link) "link" nil from to))
 
 (defun mew-attach-copy (&optional from to)
-  "Copy a file (via networks) on \".\" in attachments.
+  "Copy a file (via networks) on '.' in attachments.
 FROM must be absolute path. TO must be a file name.
 To copy a remote file, use the \"/[user@]hostname:/filepath\" syntax."
   (interactive)
   (mew-attach-duplicate (function copy-file) "copy" t from to))
 
+(defun mew-attach-link-message ()
+  "Link the message displayed in Message mode with a symbolic link
+on '.' in attachments."
+  (interactive)
+  (let ((fld-msg (mew-current-get 'message))
+	default)
+    (if fld-msg
+	(setq default (mew-expand-folder-get-msg (car fld-msg) (cdr fld-msg))))
+    (if mew-use-symbolic-link-for-forwarding
+	(mew-attach-duplicate (function mew-symbolic-link) "link" nil nil nil default)
+      (mew-attach-duplicate (function copy-file) "copy" nil nil nil default))))
+
 (defun mew-attach-find-new-file ()
-  "Open a new file into a buffer on \".\" in attachments."
+  "Open a new file into a buffer on '.' in attachments."
   (interactive)
   (if (not (mew-attach-not-line012-1))
       (message "Can't find a new file here.")
-    (let* ((nums (mew-attach-nums))
+    (let* ((nums (mew-syntax-nums))
 	   (subdir (mew-attach-expand-path mew-encode-syntax nums))
-	   (mimedir (mew-expand-folder (mew-draft-to-mime (buffer-name))))
+	   (attachdir (mew-attachdir))
 	   file filepath)
-      ;; mimedir / {subdir/} dir
+      ;; attachdir / {subdir/} dir
       (if (not (equal subdir "")) 
-	  (setq mimedir (expand-file-name subdir mimedir)))
-      ;; mimedir / file
-      (setq filepath (mew-random-filename mimedir mew-text-suffix))
+	  (setq attachdir (expand-file-name subdir attachdir)))
+      ;; attachdir / file
+      (setq filepath (mew-random-filename attachdir mew-text-suffix))
       (if (null filepath)
 	  (message "Could not make a text file, sorry.")
 	(setq file (file-name-nondirectory filepath))
@@ -428,19 +445,19 @@ To copy a remote file, use the \"/[user@]hostname:/filepath\" syntax."
 	(local-set-key "\C-c\C-q" 'mew-kill-buffer)))))
 
 (defun mew-attach-audio ()
-  "Sampling voice and insert as audio file on \".\" in attachments"
+  "Sampling voice and insert as audio file on '.' in attachments"
   (interactive)
   (if (not (mew-attach-not-line012-1))
       (message "Can't attach audio data here.")
-    (let* ((nums (mew-attach-nums))
+    (let* ((nums (mew-syntax-nums))
 	   (subdir (mew-attach-expand-path mew-encode-syntax nums))
-	   (mimedir (mew-expand-folder (mew-draft-to-mime (buffer-name))))
+	   (attachdir (mew-attachdir))
 	   file filepath recorded)
-      ;; mimedir / {subdir/} dir
+      ;; attachdir / {subdir/} dir
       (if (not (equal subdir "")) 
-	  (setq mimedir (expand-file-name subdir mimedir)))
-      ;; mimedir / file
-      (setq filepath (mew-random-filename mimedir mew-audio-suffix))
+	  (setq attachdir (expand-file-name subdir attachdir)))
+      ;; attachdir / file
+      (setq filepath (mew-random-filename attachdir mew-audio-suffix))
       (if (null filepath)
 	  (message "Could not make a file for audio, sorry.")
 	(setq file (file-name-nondirectory filepath))
@@ -450,14 +467,18 @@ To copy a remote file, use the \"/[user@]hostname:/filepath\" syntax."
 	      (progn
 		(if (not (y-or-n-p "Are you ready? "))
 		    (message "Nothing is recoded.")
-		  (message "Type C-g to finish recording...")
+		  (message
+		   (substitute-command-keys
+		    "Type '\\[keyboard-quit]' to finish recording..."))
 		  (mew-plet
 		   (apply (function call-process)
 			  (nth 0 mew-prog-audio2)
 			  nil t nil
 			  (nth 1 mew-prog-audio2)))))
 	    (quit
-	     (message "Type C-g to finish recording... done.")
+	     (message
+	      (substitute-command-keys
+	       "Type '\\[keyboard-quit]' to finish recording... done."))
 	     (setq recorded t)
 	     (mew-flet
 	      (write-region (point-min) (point-max) filepath nil 'no-msg)))))
@@ -479,7 +500,7 @@ To copy a remote file, use the \"/[user@]hostname:/filepath\" syntax."
   (interactive)
   (if (not (mew-attach-not-line0-1-dot))
       (message "Can't undo encoding here")
-    (let* ((nums (mew-attach-nums))
+    (let* ((nums (mew-syntax-nums))
 	   (syntax (mew-syntax-get-entry mew-encode-syntax nums)))
       (mew-syntax-set-privacy syntax nil)
       (mew-syntax-set-decrypters syntax nil)
@@ -494,11 +515,11 @@ To copy a remote file, use the \"/[user@]hostname:/filepath\" syntax."
   (interactive)
   (if (not (mew-attach-not-line0-1-dot))
       (message "Can't change %s here" mew-ct:)
-    (let* ((nums (mew-attach-nums))
+    (let* ((nums (mew-syntax-nums))
 	   (syntax (mew-syntax-get-entry mew-encode-syntax nums))
 	   (file (mew-syntax-get-file syntax))
 	   (ctl (mew-syntax-get-ct syntax))
-	   (ct (capitalize (car ctl)))
+	   (ct (mew-syntax-get-value ctl 'cap))
 	   cte)
       (setq ct (mew-input-type "Type for %s (%s): " file ct
 			       (if (mew-syntax-multipart-p syntax)
@@ -517,12 +538,12 @@ To copy a remote file, use the \"/[user@]hostname:/filepath\" syntax."
       (message "This commands cannot be used on this Emacs 19")
     (if (not (mew-attach-not-line0-1-dot))
 	(message "Can't specify character set here")
-      (let* ((nums (mew-attach-nums))
+      (let* ((nums (mew-syntax-nums))
 	     (syntax (mew-syntax-get-entry mew-encode-syntax nums))
 	     (file (mew-syntax-get-file syntax))
 	     (ctl (mew-syntax-get-ct syntax))
-	     (ct (car ctl))
-	     (char (mew-syntax-get-member ctl "charset"))
+	     (ct (mew-syntax-get-value ctl 'cap))
+	     (char (mew-syntax-get-param ctl "charset"))
 	     charset
 	     (case-fold-search t))
 	(if (not (string-match "text/" ct))
@@ -532,7 +553,7 @@ To copy a remote file, use the \"/[user@]hostname:/filepath\" syntax."
 					file char mew-charset-list))
 	  (if (equal charset "guess")
 	      (setq ctl (list ct))
-	    (setq ctl (list ct (format "charset=%s" charset))))
+	    (setq ctl (list ct (list "charset" charset))))
 	  (mew-syntax-set-ct syntax ctl)
 	  (mew-encode-syntax-print mew-encode-syntax))))))
 
@@ -541,7 +562,7 @@ To copy a remote file, use the \"/[user@]hostname:/filepath\" syntax."
   (interactive)
   (if (not (mew-attach-not-line0-1-dot))
       (message "Can't describe here")
-    (let* ((nums (mew-attach-nums))
+    (let* ((nums (mew-syntax-nums))
 	   (syntax (mew-syntax-get-entry mew-encode-syntax nums))
 	   (ocd (or (mew-syntax-get-cd syntax) (mew-syntax-get-file syntax))))
       (if (null cd)
@@ -556,10 +577,10 @@ To copy a remote file, use the \"/[user@]hostname:/filepath\" syntax."
   (interactive)
   (if (not (mew-attach-not-line012-1-dot))
       (message "Can't set disposition here")
-    (let* ((nums (mew-attach-nums))
+    (let* ((nums (mew-syntax-nums))
 	   (syntax (mew-syntax-get-entry mew-encode-syntax nums))
-	   (ocdp (or (mew-syntax-get-member (mew-syntax-get-cdp syntax)
-					    "filename")
+	   (ocdp (or (mew-syntax-get-param (mew-syntax-get-cdp syntax)
+					   "filename")
 		     (mew-syntax-get-file syntax))))
       (if (null cdp)
 	  (setq cdp (read-string "Filename: " ocdp)))
@@ -572,10 +593,10 @@ To copy a remote file, use the \"/[user@]hostname:/filepath\" syntax."
   (interactive)
   (if (not (mew-attach-not-line0-1-dot))
       (message "Can't encode with base64 here")
-    (let* ((nums (mew-attach-nums))
+    (let* ((nums (mew-syntax-nums))
 	   (syntax (mew-syntax-get-entry mew-encode-syntax nums))
 	   (ctl (mew-syntax-get-ct syntax))
-	   (ct (car ctl))
+	   (ct (mew-syntax-get-value ctl 'cap))
 	   (case-fold-search t))
 	(if (or (string-match "^Message/" ct) (string-match "^Multipart/" ct))
 	    (message "Can't encode with base64 here")
@@ -587,10 +608,10 @@ To copy a remote file, use the \"/[user@]hostname:/filepath\" syntax."
   (interactive)
   (if (not (mew-attach-not-line0-1-dot))
       (message "Can't encode with quoted-printable here")
-    (let* ((nums (mew-attach-nums))
+    (let* ((nums (mew-syntax-nums))
 	   (syntax (mew-syntax-get-entry mew-encode-syntax nums))
 	   (ctl (mew-syntax-get-ct syntax))
-	   (ct (car ctl))
+	   (ct (mew-syntax-get-value ctl 'cap))
 	   (case-fold-search t))
 	(if (or (string-match "^Message/" ct) (string-match "^Multipart/" ct))
 	    (message "Can't encode with quoted-printable here")
@@ -604,12 +625,14 @@ is not effective other objects. For example, JPEG is already compressed."
   (interactive)
   (if (not (mew-attach-not-line0-1-dot))
       (message "Can't apply gzip here")
-    (let* ((nums (mew-attach-nums))
+    (let* ((nums (mew-syntax-nums))
 	   (syntax (mew-syntax-get-entry mew-encode-syntax nums))
 	   (ctl (mew-syntax-get-ct syntax))
-	   (ct (car ctl))
+	   (ct (mew-syntax-get-value ctl 'cap))
 	   (case-fold-search t))
-      (if (null (mew-member-case-equal ct mew-mime-content-type-text-list))
+      (if (or (string-match "^Message/" ct)
+	      (string-match "^Multipart/" ct)
+	      (not (mew-member-case-equal ct mew-mime-content-type-text-list)))
 	  ;; never use compress to types other than text/plain or 
 	  ;; application/postscript. Indeed, compression is not
 	  ;; effective to compressed data such as jpeg.
@@ -624,12 +647,12 @@ is not effective other objects. For example, JPEG is already compressed."
    ((not (mew-attach-not-line0-1-dot))
     (message "Can't PGP sign here"))
    ((null mew-pgp-ver)
-    (message "PGP doesn't exist"))
+    (message "%s doesn't exist" mew-prog-pgp))
    (t
-    (let* ((nums (mew-attach-nums))
+    (let* ((nums (mew-syntax-nums))
 	   (syntax (mew-syntax-get-entry mew-encode-syntax nums))
 	   (privacy (mew-syntax-get-privacy syntax))
-	   (pgp-sign (list mew-ct-mls mew-ct-pgs)))
+	   (pgp-sign (nth 1 (assoc 'pgp-signature mew-privacy-database))))
       (cond
        ((>= (length privacy) 2)
 	(message "Too many marks"))
@@ -637,7 +660,7 @@ is not effective other objects. For example, JPEG is already compressed."
 	(message "Already marked."))
        (t
 	(mew-syntax-set-privacy 
-	 syntax (append privacy (list pgp-sign)))
+	 syntax (append privacy pgp-sign))
 	(mew-encode-syntax-print mew-encode-syntax)))))))
 
 (defun mew-attach-pgp-enc ()
@@ -648,13 +671,13 @@ Input decryptors' addresses."
    ((not (mew-attach-not-line0-1-dot))
     (message "Can't PGP encrypt here"))
    ((null mew-pgp-ver)
-    (message "PGP doesn't exist"))
+    (message "%s doesn't exist" mew-prog-pgp))
    (t
-    (let* ((nums (mew-attach-nums))
+    (let* ((nums (mew-syntax-nums))
 	   (syntax (mew-syntax-get-entry mew-encode-syntax nums))
 	   (privacy (mew-syntax-get-privacy syntax))
 	   (towhom (mew-header-parse-address-list mew-destination:-list))
-	   (pgp-enc (list mew-ct-mle mew-ct-pge)))
+	   (pgp-enc (nth 1 (assoc 'pgp-encryption mew-privacy-database))))
       (cond
        ((>= (length privacy) 2)
 	(message "Too many marks"))
@@ -663,11 +686,11 @@ Input decryptors' addresses."
        (t
 	;; ask towhom before set privacy for C-g
 	(if towhom
-	    (setq towhom (mew-input-address2 "To (%s): " (mew-join "," towhom)))
-	  (setq towhom (mew-input-address mew-to:)))
-	(setq towhom (mew-addrstr-parse-value-list2 towhom))
+	    (setq towhom (mew-input-address "To (%s): " (mew-join "," towhom)))
+	  (setq towhom (mew-input-address "To: ")))
+	;; towhom is already canonicalized.
 	(mew-syntax-set-privacy 
-	 syntax (append privacy (list pgp-enc)))
+	 syntax (append privacy pgp-enc))
 	(mew-syntax-set-decrypters syntax towhom)
 	(mew-encode-syntax-print mew-encode-syntax)))))))
 
@@ -680,29 +703,25 @@ Input decryptors' addresses."
   (interactive)
   (if (not (mew-attach-not-line012-1-dot))
       (message "Can't find a file here.")
-    (let* ((nums (mew-attach-nums))
+    (let* ((nums (mew-syntax-nums))
 	   (subdir (mew-attach-expand-path mew-encode-syntax nums))
-	   (mimedir (mew-expand-folder (mew-draft-to-mime (buffer-name))))
+	   (attachdir (mew-attachdir))
 	   (syntax (mew-syntax-get-entry mew-encode-syntax nums))	   
 	   (file (mew-syntax-get-file syntax))
-	   efile)
+	   efile FILE)
       (setq efile (if (equal subdir "") file (concat subdir file)))
-      (find-file (expand-file-name efile mimedir))
+      (setq FILE (expand-file-name efile attachdir))
+      (find-file FILE)
       ;; buffer switched
-      (local-set-key "\C-c\C-q" 'mew-kill-buffer))))
-
-;;
-
-(defun mew-attach-dummy ()
-  "A dummy function to clear side-effect in attachments."
-  (interactive)
-  ())
+      (local-set-key "\C-c\C-q" 'mew-kill-buffer)
+      (if (file-symlink-p FILE)
+	  (message "This file is indicated by a symbolic link. Take care.")))))
 
 (provide 'mew-attach)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996, 1997, 1998 Mew developing team.
+;; Copyright (C) 1996, 1997, 1998, 1999 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
