@@ -2,11 +2,11 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Mar 23, 1997
-;; Revised: Aug 24, 1998
+;; Revised: Aug 31, 1999
 
 ;;; Code:
 
-(defconst mew-cache-version "mew-cache.el version 0.06")
+(defconst mew-cache-version "mew-cache.el version 0.08")
 
 (require 'mew)
 
@@ -25,20 +25,28 @@
        (set-buffer (, buf))
        mew-decode-error)))
 
+(defmacro mew-cache-multi-form (buf)
+  (` (save-excursion
+       (set-buffer (, buf))
+       mew-syntax-multi-form)))
+
+(defmacro mew-cache-icon-spec (buf)
+  (` (save-excursion
+       (set-buffer (, buf))
+       mew-syntax-icon-spec)))
+
+(defmacro mew-cache-privacy-result (buf)
+  (` (save-excursion
+       (set-buffer (, buf))
+       mew-syntax-privacy-result)))
+
 (defvar mew-cache nil
-      "Mail cache. (old ... new) order alist with association
- ((\"+folder\" . \"message\") . cache-buffer)"
-      )
+  "A list of decoded messages cache. 
+The (new ... old) order of ((\"+folder\" . \"message\") . cache-buffer)")
 
 (defmacro mew-cache-buffer-get (entry)
   (` (cdr (, entry))))
 
-(defmacro mew-cache-folder-get (entry)
-  (` (car (car (, entry)))))
-
-(defmacro mew-cache-message-get (entry)
-  (` (cdr (car (, entry)))))
-
 (defmacro mew-cache-entry-make (fld-msg buf)
   (` (cons (, fld-msg) (, buf))))
 
@@ -47,70 +55,70 @@
   (` (mew-cache-buffer-get (assoc (, fld-msg) mew-cache))))
 
 (defun mew-cache-sort (entry)
-  (let* ((pointer (cons nil mew-cache))
-	 (top pointer))
-    (while (cdr pointer)
-      (if (equal (car (cdr pointer)) entry)
-	  (setcdr pointer (cdr (cdr pointer)))
-	(setq pointer (cdr pointer))))
-    (setcdr pointer (list entry))
-    (setq mew-cache (cdr top))))
+  (setq mew-cache (cons entry (delete entry mew-cache))))
 
 (defun mew-cache-add (fld-msg)
-  "Rutern associating buffer"
+  "Adding (fld-msg . buf) to the top of 'mew-cache'.
+Returning its cache buffer."
   (let ((len (length mew-cache))
-	(buf nil))
-    (if (>= len mew-cache-size)
-	(prog1
-	    (setq buf (mew-cache-buffer-get (car mew-cache)))
-	  (setq mew-cache (nconc (cdr mew-cache)
-				  (list (mew-cache-entry-make fld-msg buf))))
-	  )
-      (prog1
-	  (setq buf (get-buffer-create 
-		     (concat mew-buffer-cache (int-to-string len))))
-	(setq mew-cache (nconc mew-cache
-				(list (mew-cache-entry-make fld-msg buf))))
-	))
-    ))
+	buf)
+    (if (< len mew-cache-size)
+	(setq buf (get-buffer-create (format "%s%d" mew-buffer-cache len)))
+      (setq buf (mew-cache-buffer-get (nth (1- len) mew-cache)))
+      (setcdr (nthcdr (- len 2) mew-cache) nil))
+    (setq mew-cache (cons (mew-cache-entry-make fld-msg buf) mew-cache))
+    buf))
 
 (defun mew-cache-delete ()
   "Delete the most recent cache entry."
-  (setq mew-cache (nreverse (cdr (nreverse mew-cache)))))
+  (let ((buf (mew-cache-buffer-get (car mew-cache))))
+    ;; must preserve the buffer itself because the buffer creation
+    ;; depends on the length of mew-cache.
+    (setq mew-cache (nconc (cdr mew-cache)
+			   (list (mew-cache-entry-make nil buf))))))
 
-(defun mew-cache-attribute-get (file)
-  ;; 5th is modified time
-  ;; 7th is file size
-  (let ((attr (file-attributes file)))
-    (list (nth 5 attr)
-	  (nth 7 attr))))
+
+(defmacro mew-cache-attribute-get (file)
+  (` (list (mew-file-get-time (, file)) (mew-file-get-size (, file)))))
        
-(defun mew-cache-message (fld msg)
-  (let ((hit (mew-cache-hit (cons fld msg)))
-	(decode nil))
+(defun mew-cache-message (fld msg &optional force)
+  (let* ((fld-msg (cons fld msg))
+	 (hit (mew-cache-hit fld-msg))
+	 (file (mew-expand-folder fld msg))
+	 (decode nil))
     (if hit
 	(save-excursion
-	  (mew-cache-sort (mew-cache-entry-make (cons fld msg) hit))
+	  (mew-cache-sort (mew-cache-entry-make fld-msg hit))
 	  (set-buffer hit)
-	  (if (and (not (mew-folder-newsp fld))
-		   (not (equal mew-cache-attribute ;; buffer-local 
-			       (mew-cache-attribute-get
-				(mew-expand-folder fld msg)))))
+	  (if (or (and (mew-folder-localp fld)
+		       (not (equal mew-cache-attribute ;; buffer-local 
+				   (mew-cache-attribute-get file))))
+		  (and force mew-decode-not-decrypted)
+		  (and force mew-decode-error))
 	      ;; cache is invalid
 	      (setq decode t)))
-      (setq hit (mew-cache-add (cons fld msg)))
+      (setq hit (mew-cache-add fld-msg))
       (setq decode t))
     (if decode
 	(condition-case nil
 	    (save-excursion
 	      (set-buffer hit)
 	      ;; in cache buffer
-	      (mew-decode fld msg))
+	      (setq mew-cache-folder fld)
+	      (setq mew-cache-message-number msg)
+	      (if (mew-folder-localp fld)
+		  (setq mew-cache-attribute (mew-cache-attribute-get file))
+		(setq mew-cache-attribute nil))
+	      (if force
+		  (let ((mew-header-max-length nil)
+			(mew-header-max-depth nil))
+		    (mew-decode fld msg))
+		(mew-decode fld msg)))
 	  (quit
 	   (mew-cache-delete)
-	   (error "MIME decoding is quitted."))))
-    hit ;; retrun value
-    ))
+	   (message "MIME decoding for %s/%s is quitted." fld msg)
+	   nil))) ;; will not be used
+    hit)) ;; retrun value
 
 (defun mew-cache-clean-up ()
   "A function to flush all decoded messages in cache list."
@@ -118,9 +126,8 @@
   (mew-decode-syntax-delete)
   (let ((n 0))
     (while (< n mew-cache-size)
-      (mew-kill-buffer (concat mew-buffer-cache (int-to-string n)))
-      (setq n (1+ n))
-      ))
+      (mew-kill-buffer (format "%s%d" mew-buffer-cache n))
+      (setq n (1+ n))))
   (mew-current-set 'cache nil)
   (mew-current-set 'message nil)
   (mew-current-set 'part nil)
@@ -132,7 +139,7 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997, 1998 Mew developing team.
+;; Copyright (C) 1997, 1998, 1999 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
