@@ -2,29 +2,23 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: May 30, 1997
-;; Revised: Aug 24, 1998
+;; Revised: Aug 30, 1999
 
 ;;; Code:
 
-(defconst mew-complete-version "mew-complete.el version 0.03")
+(defconst mew-complete-version "mew-complete.el version 0.04")
 
 (require 'mew)
 
-;;
-;; Switching completion in Draft
-;;
+;; hoping the functions here are free from marker problem
+;; because it inserts any chars before \n. It is important
+;; to use the position's color.
+(fset 'mew-complete-insert (symbol-function 'insert))
 
-(defun mew-draft-header-comp ()
-  "Complete and expand address aliases. 
-First alias key is completed. When completed solely or the @ character
-is inserted before the cursor, the alias key is expanded to its value."
-  (interactive)
-  (if (mew-draft-on-field-p)
-      (mew-complete-field)
-    (let ((func (mew-draft-on-value-p mew-field-completion-switch)))
-      (if func 
-	  (funcall (cdr func))
-	(tab-to-tab-stop))))) ;; default keybinding
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Low level functions
+;;;
 
 (defun mew-draft-on-field-p ()
   (if (bolp)
@@ -32,29 +26,23 @@ is inserted before the cursor, the alias key is expanded to its value."
 	  t
 	(save-excursion
 	  (forward-line -1)
-	  (if (looking-at ".*,[ \t]?$") nil t)
-	  )
-	)
+	  (if (looking-at ".*,[ \t]?$") nil t)))
     (let ((pos (point)))
       (save-excursion
 	(beginning-of-line)
 	(if (looking-at mew-lwsp)
 	    nil
-	  (if (search-forward ":" pos t) nil t))
-	))
-    ))
+	  (if (search-forward ":" pos t) nil t))))))
       
 (defun mew-draft-on-value-p (switch)
   (save-excursion
     (beginning-of-line)
     (while (and (< (point-min) (point))	(looking-at mew-lwsp))
-      (forward-line -1)
-      )
+      (forward-line -1))
     (if (looking-at "\\([^:]*:\\)")
 	(mew-assoc-match (mew-match 1) switch 0)
-      nil) ;; what a case reach here?
-    ))
-      
+      nil))) ;; what a case reachs here?
+  
 ;;
 ;; Window management for completion candidates
 ;;
@@ -75,13 +63,10 @@ is inserted before the cursor, the alias key is expanded to its value."
     ;; buffer is displayed or not.
     (if (get-buffer-window mew-buffer-completions)
 	(set-window-configuration mew-complete-window-config))
-    (setq mew-complete-window-config nil)
-    )
+    (setq mew-complete-window-config nil))
   (and (get-buffer mew-buffer-completions)
        (kill-buffer mew-buffer-completions))
-  (setq mew-complete-candidates nil)
-  )
-
+  (setq mew-complete-candidates nil))
 
 (defun mew-complete-window-show (all)
   (or mew-complete-window-config
@@ -100,56 +85,181 @@ is inserted before the cursor, the alias key is expanded to its value."
       (display-completion-list all))))
 
 (defun mew-complete-backscroll ()
+  "Backscroll the *Completion* buffer."
   (interactive)
   (let* ((win (get-buffer-window mew-buffer-completions))
 	 (height (and win (window-height win))))
     (and win (scroll-other-window (- 3 height)))))
 
-;;
-;; Completion function
-;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Completion function: C-i
+;;;
+
+(defun mew-draft-header-comp ()
+  "Complete and expand address short names.
+First, a short name is completed. When completed solely or the @ character
+is inserted before the cursor, the short name is expanded to its address."
+  (interactive)
+  (if (mew-draft-on-field-p)
+      (mew-complete-field)
+    (let ((func (mew-draft-on-value-p mew-field-completion-switch)))
+      (if func 
+	  (funcall (cdr func))
+	(tab-to-tab-stop))))) ;; default keybinding
 
 (defun mew-complete-field ()
+  "Field complete function."
   (interactive)
   (let ((word (mew-delete-key))) ;; capitalized
     (if (null word)
 	(mew-complete-window-show mew-fields)
       (mew-complete
        word
-       (mapcar (lambda (x) (list (concat (capitalize x) " "))) mew-fields)
+       (mapcar (function (lambda (x) (list (concat (capitalize x) " "))))
+	       mew-fields)
        "field"
        nil))))
 
 (defun mew-complete-folder ()
+  "Folder complete function for Fcc:."
   (interactive)
   (let ((word (mew-delete-backward-char)))
     (if (null word)
-	(mew-complete-window-show (list "+" "="))
+	(if mew-use-imap
+	    (mew-complete-window-show (list "+" "=" "%"))
+	  (mew-complete-window-show (list "+" "=")))
       (mew-complete word mew-folder-alist "folder" nil))))
 
 (defun mew-complete-address ()
+  "Complete and expand an address short name.
+First alias key is completed. When completed solely or the @ character
+is inserted before the cursor, the short name is expanded to its address."
   (interactive)
   (let ((word (mew-delete-backward-char)))
     (if (null word)
 	(tab-to-tab-stop)
-      (mew-complete word mew-alias-alist "alias" ?@)))) ;; use cdr
+      (if (string-match "@." word)
+	  (mew-complete-insert (or (mew-alias-next word) word))
+	(mew-complete
+	 word mew-addrbook-alist "alias" ?@ nil nil
+	 (function mew-addrbook-alias-get) 
+	 (function mew-addrbook-alias-hit))))))
 
 (defun mew-complete-config ()
+  "Complete function for Config:."
   (interactive)
-  (let ((word (mew-delete-value ",")))
-    (if (null word)
-	(tab-to-tab-stop)
+  (let ((word (or (mew-delete-value ",") "")))
+    (mew-complete
+     word
+     (mew-slide-pair mew-config-list)
+     "mew-config-list"
+     nil)))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Circular completion: C-cC-i
+;;;
+
+(defun mew-draft-circular-comp ()
+  "Switch function for circular complete functions."
+  (interactive)
+  (let ((func (mew-draft-on-value-p mew-field-circular-completion-switch)))
+    (if func
+	(funcall (cdr func))
+      (message "No circular completion here"))))
+
+(defun mew-circular-complete-domain ()
+  "Circular completion of domains for To:, Cc:, etc.
+If the @ character does not exist, the first value of
+mew-mail-domain-list is inserted. If exists, the next value of 
+mew-mail-domain-list concerned with the string between @ and 
+the cursor is inserted."
+  (interactive)
+  (let ((word (mew-delete-backward-char "@")))
+    (cond
+     ((equal word nil) ;; @ doesn't exist.
+      (if (null mew-mail-domain-list)
+	  (message "For domain circular completion, set mew-mail-domain-list")
+	(mew-complete-insert "@")
+	(mew-complete-insert (car mew-mail-domain-list))
+	(mew-complete-window-delete)))
+     ((equal word t) ;; just after @
+      (if (null mew-mail-domain-list)
+	  (message "For domain circular completion, set mew-mail-domain-list")
+	(mew-complete-insert (car mew-mail-domain-list))
+	(mew-complete-window-delete)))
+     (t
+      ;; can't use mew-get-next since completion is necessary sometime.
       (mew-complete
        word
-       (mew-slide-pair mew-config-list)
-       "mew-config-list"
-       nil))))
+       (mew-slide-pair mew-mail-domain-list)
+       "domain"
+       t)) ;; use cdr
+     )))
+
+(defun mew-circular-complete (msg clist cname &optional here)
+  "General circular complete function to call mew-complete."
+  (interactive)
+  (let ((str (mew-delete-value here)))
+    (if (null str)
+	(if (car clist)
+	    (mew-complete-insert (car clist))
+	  (message "For circular completion, set %s" cname))
+      (mew-complete
+       str
+       (mew-slide-pair clist)
+       msg
+       t)))) ;; use cdr
+
+(defun mew-circular-complete-from ()
+  "Circular complete function for From:."
+  (interactive)
+  (mew-circular-complete "from" mew-from-list "mew-from-list"))
+
+(defun mew-circular-complete-config ()
+  "Circular complete function for Config:."
+  (interactive)
+  (mew-circular-complete "config" mew-config-list "mew-config-list" ","))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Expansion : M-C-i
+;;;
+
+(defun mew-draft-expand ()
+  "Switch function for expand functions."
+  (interactive)
+  (let ((func (mew-draft-on-value-p mew-field-expansion-switch)))
+    (if func
+	(funcall (cdr func))
+      (message "No expansion here"))))
+
+(defun mew-expand-address ()
+  "Address expansion fuction for To:, Cc:, etc.
+'user@domain' will be expands 'name <user@domain>' if
+the name exists."
+  (interactive)
+  (let ((word (mew-delete-backward-char)) func name)
+    (if (null word)
+	(message "No address here")
+      (setq func (mew-addrbook-func mew-addrbook-for-address-expansion))
+      (if (null func)
+	  (mew-complete-insert word)
+	(setq name (funcall func word))
+	(mew-complete-insert (if name (format "%s <%s>" name word) word))))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Other completion stuff
+;;;
 
 ;; dummy
 (defvar mew-ext-host "")
 (defvar mew-ext-user "")
 
 (defun mew-complete-rfile ()
+  "Complete a remote file."
   (interactive)
   (let* ((path-file (mew-delete-file))
 	 (path (car path-file))
@@ -165,6 +275,7 @@ is inserted before the cursor, the alias key is expanded to its value."
      (function mew-ext-file-name-all-completions))))
 
 (defun mew-complete-pick-pattern ()
+  "Complete pick patterns."
   (interactive)
   (let* ((pat (mew-delete-pattern))
 	 (clist (append '("(" "!")
@@ -179,113 +290,53 @@ is inserted before the cursor, the alias key is expanded to its value."
        nil))))
 
 (defun mew-complete-sort-key ()
+  "Complete sort keys."
   (interactive)
   (let* ((word (mew-delete-line))
 	 field alist)
     (if (string-match ":" word)
 	(progn
-	  ;; If WORD contains `:', change alist for completion.
+	  ;; If WORD contains ':', change alist for completion.
 	  (setq field (car (mew-split word ?:)))
 	  (setq alist 
-		(mapcar (lambda (str) (list (concat field ":" str)))
+		(mapcar (function (lambda (str) (list (concat field ":" str))))
 			mew-sort-modes)))
       ;; Otherwise, alist is mew-sort-key-alist itself.
       (setq alist mew-sort-key-alist))
     (mew-complete word alist "sort key" nil)))
 
-;;
-;; Circular completion: C-cC-t
-;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Hart function for completions
+;;;
 
-;; hoping this function is free from marker problem
-;; because it inserts any chars before \n. It is important
-;; to use the position's color.
-(fset 'mew-complete-insert (symbol-function 'insert))
+(fset 'mew-complete-hit (symbol-function 'assoc))
 
-(defun mew-draft-circular-comp ()
-  "Circular completion of domains.
-If the @ character does not exist, the first value of
-mew-mail-domain-list is inserted. If exists, the next value of 
-mew-mail-domain-list concerned with the string between @ and 
-the cursor is inserted."
-  (interactive)
-  (let ((func (mew-draft-on-value-p mew-field-circular-completion-switch)))
-    (if func
-	(funcall (cdr func))
-      (message "No circular completion here"))))
+(defun mew-complete-get (key alist)
+  (cdr (mew-complete-hit key alist)))
 
-(defun mew-circular-complete-domain ()
-  (interactive)
-  (let ((word (mew-delete-backward-char "@")))
-    (cond
-     ((equal word nil) ;; @ doesn't exist.
-      (if (null mew-mail-domain-list)
-	  (message "For domain circular completion, set mew-mail-domain-list")
-	(mew-complete-insert "@")
-	(mew-complete-insert (car mew-mail-domain-list))
-	(mew-complete-window-delete))
-      )
-     ((equal word t) ;; just after @
-      (if (null mew-mail-domain-list)
-	  ()
-	(mew-complete-insert (car mew-mail-domain-list))
-	(mew-complete-window-delete))
-      )
-     (t
-      (mew-complete
-       word
-       (mew-slide-pair mew-mail-domain-list)
-       "domain"
-       t) ;; use cdr
-      )
-     )
-    ))
-
-(defun mew-circular-complete (msg clist cname &optional here)
-  (interactive)
-  (let ((str (mew-delete-value here)))
-    (if (null str)
-	(if (car clist)
-	    (mew-complete-insert (car clist))
-	  (message "For circular completion, set %s" cname))
-      (mew-complete
-       str
-       (mew-slide-pair clist)
-       msg
-       t) ;; use cdr
-      )))
-
-(defun mew-circular-complete-from ()
-  (interactive)
-  (mew-circular-complete "from" mew-from-list "mew-from-list"))
-
-(defun mew-circular-complete-config ()
-  (interactive)
-  (mew-circular-complete "config" mew-config-list "mew-config-list" ","))
-
-;;
-;; Hart function for completions
-;;
-
-(defun mew-complete (WORD ALIST MSG EPAND-CHAR &optional TRY ALL)
+(defun mew-complete (WORD ALIST MSG EXPAND-CHAR &optional TRY ALL GET HIT)
   (let* ((ftry (or TRY (function try-completion)))
 	 (fall (or ALL (function all-completions)))
+	 (fget (or GET (function mew-complete-get)))
+	 (fhit (or HIT (function mew-complete-hit)))
 	 (cmp (funcall ftry WORD ALIST))
 	 (all (funcall fall WORD ALIST))
-	 (len (length WORD)))
+	 (len (length WORD))
+	 subkey)
     (cond
      ;; already completed
      ((eq cmp t)
-      (if EPAND-CHAR
-	  (mew-complete-insert (cdr (assoc WORD ALIST))) ;; use cdr
+      (if EXPAND-CHAR ;; may be "t"
+	  (mew-complete-insert (funcall fget WORD ALIST)) ;; use cdr
 	(mew-complete-insert WORD)) ;; use car
       (mew-complete-window-delete))
      ;; EXPAND
-     ((and (mew-characterp EPAND-CHAR)
-	   (char-equal (aref WORD (1- len)) EPAND-CHAR)
-	   (assoc (substring WORD 0 (1- len)) ALIST))
-      (mew-complete-insert (cdr (assoc (substring WORD 0 (1- len)) ALIST)))
-      ;; use cdr
+     ((and (mew-characterp EXPAND-CHAR)
+	   (char-equal (aref WORD (1- len)) EXPAND-CHAR)
+	   (setq subkey (substring WORD 0 (1- len)))
+	   (funcall fhit subkey ALIST))
+      (mew-complete-insert (funcall fget subkey ALIST)) ;; use cdr
       (mew-complete-window-delete))
      ;; just one candidate
      ((equal 1 (length all))
@@ -297,21 +348,24 @@ the cursor is inserted."
      ;; two or more candidates
      ((stringp cmp) ;; (length all) > 1
       (mew-complete-insert cmp)
-      (mew-complete-window-show all))
+      (mew-complete-window-show all)
+      (if (and EXPAND-CHAR (funcall fhit cmp ALIST))
+	  (message
+	   (substitute-command-keys
+	    "To expand '%s', type '%c' then '\\<mew-draft-header-map>\\[mew-draft-header-comp]'.")
+	   cmp EXPAND-CHAR)))
      ;; no candidate
      (t
       (mew-complete-insert WORD)
       ;;(mew-complete-window-delete)
       (if (window-minibuffer-p (get-buffer-window (current-buffer)))
 	  (mew-temp-minibuffer-message (concat " No matching " MSG))
-	(message "No matching %s" MSG))
-      )
-     )
-    ))
+	(message "No matching %s" MSG))))))
 
-;;
-;; Minibuf magic
-;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Minibuf magic
+;;;
 
 (defun mew-temp-minibuffer-message (m)
   (let ((savemax (point-max)))
@@ -321,9 +375,7 @@ the cursor is inserted."
     (let ((inhibit-quit t))
       (sit-for 2)
       (delete-region savemax (point-max))
-      (if quit-flag (setq quit-flag nil	unread-command-events 7))
-      )))
-
+      (if quit-flag (setq quit-flag nil	unread-command-events 7)))))
 
 ;;
 ;; Extracting completion key
@@ -415,8 +467,7 @@ the cursor is inserted."
      (t
       (while (cdr x)
 	(setq ret (cons (cons (nth 0 x) (nth 1 x)) ret))
-	(setq x (cdr x))
-	)
+	(setq x (cdr x)))
       (setq ret (cons (cons (car x) first) ret))
       (nreverse ret)))))
 
@@ -424,7 +475,7 @@ the cursor is inserted."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997, 1998 Mew developing team.
+;; Copyright (C) 1997, 1998, 1999 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
