@@ -2,11 +2,11 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Oct  2, 1996
-;; Revised: Aug 24, 1998
+;; Revised: Aug 30, 1999
 
 ;;; Code:
 
-(defconst mew-encode-version "mew-encode.el version 0.22")
+(defconst mew-encode-version "mew-encode.el version 0.23")
 
 (require 'mew)
 
@@ -21,99 +21,249 @@
 	(cons mew-xg  '("-g" "-t"))))
 
 (defvar mew-encode-multipart-encrypted-switch
-  '(("application/pgp-encrypted"  . mew-pgp-encrypt)
-;;    ("application/moss-keys"      . mew-moss-encrypt)
-    ))
+  '(("application/pgp-encrypted" . mew-pgp-encrypt)))
 
 (defvar mew-encode-multipart-signed-switch
-  '(("application/pgp-signature"   . mew-pgp-sign)
-;;    ("application/moss-signature"  . mew-moss-sign)
-;;    ("application/pkcs7-signature" . mew-smime-sign)
-    ))
+  '(("application/pgp-signature" . mew-pgp-sign)))
 
 ;;;
 ;;;
 ;;;
 
+(defun mew-draft-remove-illegal-null-lines ()
+  (if (mew-header-end)
+      (save-excursion
+	(save-restriction
+	  (goto-char (mew-header-end))
+	  (if (not (bolp)) (insert "\n"))
+	  (narrow-to-region (point-min) (1- (mew-header-end)))
+	  (goto-char (point-min))
+	  (mew-elet
+	   (while (re-search-forward "^$" nil t)
+	     (delete-char 1)))))))
+
 (defun mew-draft-ask-subject ()
-  (if mew-ask-subject
+  (if (and mew-ask-subject
+	   (not (mew-header-existp mew-subj:)))
+      ;; value is null or Subject: doesn't exsit
       (save-excursion
-	(goto-char (point-min))
-	(re-search-forward (concat "^" mew-subj:) (mew-header-end) t 1)
-	(if (looking-at "\\s-*$")
-	    (progn
-	      (replace-match " " nil t)
-	      (mew-header-insert (read-string (concat mew-subj: " "))))))))
+	(save-restriction
+	  (narrow-to-region (point-min) (1- (mew-header-end)))
+	  (goto-char (point-min))
+	  (if (re-search-forward (concat "^\\(" mew-subj: "\\)[ \t]*$") nil t)
+	      (progn
+		(replace-match "\\1 ")
+		(insert (read-string (concat mew-subj: " ")))))))))
 
-(defun mew-draft-make-message ()
-  "Make a MIME message. Charset guess, mapping directory structure 
+(defun mew-draft-ask-newsgroups ()
+  (if (and mew-ask-newsgroups
+	   (mew-header-existp mew-newsgroups:))
+      (if (y-or-n-p "Do you want to post to NetNews? ")
+	  ()
+	(mew-header-delete-lines (list mew-newsgroups:)))))
+
+(defun mew-draft-make-message (&optional privacy)
+  "Make a MIME message. Guess charsets, convert the directory structure 
 to multipart, and so on."
   (interactive)
   (widen)
-  (mew-draft-ask-subject)
-  (if mew-config-insert-when-composed
-      (mew-draft-insert-config))
-  (if (mew-header-existp mew-ct:)
+  (run-hooks 'mew-make-message-hook)
+  (if (not (mew-header-p))
       (progn
 	(ding)
-	(message "%s already exists!" mew-ct:))
-    (message "Making a MIME letter ...")
-    (if (mew-attach-p)
-	(let ((mew-inherit-signer
-	       (or (mew-header-parse-address mew-from:) mew-mail-address)))
-	  ;; delete multipart syntax
-	  (mew-attach-clear)
-	  (if (mew-attach-valid-p)
-	      (mew-draft-make-multi)
-	    (mew-draft-make-single)))
-      (mew-draft-make-single))
-    (mew-draft-make-header)
-    (message "Making a MIME letter ... done")))
+	(message "Already made!"))
+    (condition-case nil
+	(let (multip type receivers)
+	  (mew-draft-remove-illegal-null-lines)
+	  (mew-draft-ask-subject)
+	  (mew-draft-ask-newsgroups)
+	  (if mew-config-insert-when-composed
+	      (mew-draft-insert-config))
+	  (goto-char (mew-header-end))
+	  (forward-line) ;; necessary for PGP
+	  (setq receivers (mew-encode-canonicalize-address-region
+			   (point-min) (point) mew-destination:-list))
+	  (message "Making a MIME letter ...")
+	  (if (mew-header-existp mew-ct:) ;; re-editing multipart
+	      (progn
+		(mew-draft-make-backup 'single)
+		(mew-draft-make-header 'addsep))
+	    (if (not (mew-attach-p))
+		(setq mew-encode-syntax (mew-encode-syntax-single "text-file"))
+	      (mew-attach-clear)
+	      (if (mew-attach-valid-p)
+		  (setq multip t)
+		(setq mew-encode-syntax (mew-encode-syntax-single "text-file"))
+		))
+	    (mew-draft-make-backup (not multip))
+	    ;; save syntax before setting privacy
+	    (if (or (mew-syntax-get-privacy mew-encode-syntax) ;; specified
+		    (and (null privacy) mew-draft-privacy-error)) ;; bypass
+		()
+	      (cond
+	       (privacy
+		(setq type privacy))
+	       (mew-draft-protect-privacy-type
+		(setq type mew-draft-protect-privacy-type))
+	       ((and mew-protect-privacy-encrypted mew-draft-encrypted-p)
+		(setq type mew-protect-privacy-encrypted-type))
+	       (mew-protect-privacy-always
+		(setq type mew-protect-privacy-always-type)))
+	      (mew-syntax-set-privacy
+	       mew-encode-syntax
+	       (nth 1 (assoc type mew-privacy-database)))
+	      ;; receivers are ignored when signing
+	      (mew-syntax-set-decrypters mew-encode-syntax receivers))
+	    (let ((mew-inherit-signer (mew-get-my-address)))
+	      (if multip
+		  (mew-draft-make-multi)
+		(mew-draft-make-single)))
+	    (mew-draft-make-header))
+	  (save-buffer)
+	  (setq mew-encode-syntax nil) ;; for undo
+	  (setq buffer-undo-list nil)
+	  (message "Making a MIME letter ... done"))
+      (mew-draft-undo)))) ;; may not work due to timing
 
-(defun mew-draft-make-header ()
-  ;; Removing the header separator and fix the marker.
-  ;; Then RFC 2047 header encoding.
-  (mew-header-clear) ;; this is unavoidable
-  (goto-char (mew-header-end))
-  ;; delete header separator
-  (let ((beg (point)) (inhibit-read-only t))
-    (forward-line)
-    (delete-region beg (point)))
-  (mew-header-clear) ;; this is unavoidable
-  (goto-char (mew-header-end))
-  (mew-header-encode-region (point-min) (point))
-  (mew-highlight-header)
-  (goto-char (mew-header-end))
-  (mew-header-clear) ;; really clear
+(defun mew-encode-canonicalize-address-region (BEG END fields)
+  (let ((case-fold-search t)
+	(regex (mew-make-field-regex fields))
+	start vals val addrs addr ret insl ins prefix suffix)
+    (save-excursion
+      (save-restriction
+	(narrow-to-region BEG END)
+	(goto-char (point-min))
+	(while (re-search-forward regex nil t)
+	  (setq start (match-end 0))
+	  (forward-line)
+	  (while (looking-at mew-lwsp)
+	    (delete-backward-char 1)
+	    (forward-line))
+	  (setq val (mew-buffer-substring start (1- (point))))
+	  (delete-region start (1- (point)))
+	  (backward-char 1)
+	  ;;
+	  (setq vals (mapcar (function mew-chop) (mew-split val ?, ?: ?\;)))
+	  (while vals
+	    (setq val (car vals))
+	    (setq vals (cdr vals))
+	    (setq ins nil addrs nil)
+	    (cond
+	     ((string-match "^\\([^:]+:\\)\\([^;]+\\);$" val)
+	      (setq prefix (mew-match 1 val))
+	      (setq addr (mew-match 2 val))
+	      (setq addr (mapcar (function mew-chop) (mew-split addr ?,)))
+	      (while addr
+		(setq addrs
+		      (nconc addrs (mew-addrstr-expand-alias (car addr))))
+		(setq addr (cdr addr)))
+	      (setq ins (mapconcat (function identity) addrs ","))
+	      (setq ins (list (concat prefix ins ";"))))
+	     ((and (setq addr (mew-addrstr-parse-address val))
+		   (string-match (concat "\\(.*<\\)" (regexp-quote addr) "\\(>.*\\)")
+				 val))
+	      (setq prefix (mew-match 1 val))
+	      (setq suffix (mew-match 2 val))
+	      (setq addr (mew-addrstr-append-domain addr))
+	      (setq addrs (list addr))
+	      (setq ins (list (concat prefix addr suffix))))
+	     (t
+	      (setq addrs (mew-addrstr-expand-alias val))
+	      (setq ins (copy-sequence addrs))))
+	    (setq insl (nconc insl ins))
+	    (setq ret (nconc ret addrs)))
+	  (insert " " (mapconcat (function identity) insl ", "))
+	  (setq insl nil)
+	  (forward-line))))
+    ret))
+
+(defun mew-draft-make-header (&optional addsep)
+  (if (mew-header-existp mew-mv:)
+      ()
+    (goto-char (mew-header-end))
+    (mew-header-insert mew-mv: mew-mv:-num))
+  (mew-header-encode-region (point-min) (mew-header-end))
+  (if addsep ;; reedit
+      (progn
+	;; To:
+	;; Content-*
+	;; ---
+	(mew-header-clear) ;; mew-header-p returns nil
+	;; To:
+	;; Content-*
+	(insert "\n"))
+    ;; To:
+    ;; ----
+    ;; Content-*
+    (mew-header-clear) ;; mew-header-p returns nil
+    ;; To:
+    ;; Content-*
+    )
+  (mew-header-goto-end)
+  (mew-highlight-header-region (point-min) (point))
   (set-window-start (get-buffer-window (current-buffer)) (point-min))
   (mew-draft-toolbar-update))
 
 (defun mew-draft-make-single ()
-  (mew-draft-make-backup 'single)
-  (goto-char (mew-header-end))
+  (goto-char (mew-header-end)) ;; due to illegal null lines in the header
   (forward-line)
-  (mew-encode-singlepart (mew-encode-syntax-single "text-file") nil nil t))
+  (mew-encode-singlepart mew-encode-syntax nil nil t))
+
+;;
+;; backup and undo
+;;
 
 (defun mew-draft-make-backup (&optional single)
   ;; back up the draft and its syntax
-  (let* ((path (mew-expand-folder (mew-draft-to-mime (buffer-name))))
-	 (backup-file (expand-file-name mew-draft-backup-file path))
-	 (syntax-file (expand-file-name mew-draft-syntax-file path))
-	 (syntax mew-encode-syntax) ;; mew-encode-syntax is buffer local
-	 (find-file-hooks nil))
-    (if (not (file-exists-p path)) (mew-make-directory path)) ;;for single part
-    (mew-frwlet
-     mew-cs-noconv mew-cs-draft
+  (let* ((attachdir (mew-attachdir))
+	 (backup-file (expand-file-name mew-draft-backup-file attachdir))
+	 (syntax-file (expand-file-name mew-draft-syntax-file attachdir))
+	 (syntax mew-encode-syntax)) ;; mew-encode-syntax is buffer local
+    (if (not (file-exists-p attachdir)) (mew-make-directory attachdir)) ;;for single part
+    (mew-frwlet mew-cs-dummy mew-cs-draft
      (write-region (point-min) (point-max) backup-file nil 'no-msg))
     (if (not single)
 	(save-excursion
-	  (set-buffer (find-file-noselect syntax-file))
-	  (mew-erase-buffer)
+	  (mew-set-buffer-tmp)
 	  (prin1 syntax (current-buffer)) ;; different buffer, so use syntax
 	  (terpri (current-buffer))
-	  (save-buffer)
-	  (message "") ;; flush echo area
-	  (kill-buffer (current-buffer))))))
+	  (mew-frwlet mew-cs-dummy mew-cs-draft
+	    (write-region (point-min) (point-max) syntax-file nil 'no-msg))))))
+
+(defun mew-draft-undo ()
+  "Get back to the draft before making MIME message."
+  (interactive)
+  (mew-elet
+   (let* ((attachdir (mew-attachdir))
+	  (backup-file (expand-file-name mew-draft-backup-file attachdir))
+	  (syntax-file (expand-file-name mew-draft-syntax-file attachdir))
+	  (syntax nil))
+     (if (not (file-exists-p backup-file))
+	 (message "Can't undo")
+       (mew-erase-buffer)
+       (mew-frwlet mew-cs-draft mew-cs-dummy
+	(insert-file-contents backup-file))
+       (delete-file backup-file)
+       ;;
+       (mew-header-clear) ;; erase the old header separator
+       (mew-header-prepared)
+       (if (not (file-exists-p syntax-file))
+	   () ;; single
+	 (save-excursion
+	   (mew-frwlet mew-cs-draft mew-cs-dummy
+	     (find-file-read-only syntax-file))
+	   (goto-char (point-min))
+	   (setq syntax (read (current-buffer)))
+	   (kill-buffer (current-buffer)))
+	 (setq mew-encode-syntax syntax) ;; buffer local
+	 (mew-draft-prepare-attachments)
+	 (delete-file syntax-file))
+       (mew-draft-toolbar-update)
+       (setq buffer-undo-list nil)))))
+
+;;
+;;
+;;
 
 ;; xxx will be obsoleted?
 (defun mew-encode-escape-region (beg end)
@@ -129,16 +279,22 @@ to multipart, and so on."
 (defun mew-encode-text-magic (infile ctl cte)
   ;; return (file charset cte encopts)
   ;; If infile is 't', target is buffered.
-  (let* ((charset (mew-syntax-get-member ctl "charset"))
+  (let* ((charset (mew-syntax-get-param ctl "charset"))
 	 ;; charset exists only when file is not buffered.
 	 encopts outfile)
     (if (and charset cte)
 	(list infile charset cte (mew-encode-get-topts cte)) ;; return
-      (if (stringp infile)
+      (if (and (stringp infile) (file-readable-p infile))
 	  ;; Even if charset is specified, we need to convert it to
 	  ;; transfer charset.
 	  (mew-frwlet
-	   (or (mew-charset-to-cs charset) mew-cs-infile) mew-cs-noconv
+	   ;; temporary solution for Emacs 20.3. yes, very ad-hoc
+	   (or (mew-charset-to-cs charset)
+	       (if (equal infile (expand-file-name mew-draft-coverpage
+						   (mew-attachdir)))
+		   mew-cs-draft
+		 mew-cs-infile))
+	   mew-cs-dummy
 	   ;; Input the text file converting its charset to the internel.
 	   ;; If charset is specified, use it.
 	   ;; If mew-cs-infile is *autoconv*/'autodetect, charset is guessed
@@ -160,7 +316,7 @@ to multipart, and so on."
 	(message "Cannot guess charset. Confirm charset later!!")
 	(ding)
 	(sit-for 1)
-	(setq charset "us-ascii"))
+	(setq charset mew-us-ascii))
       ;; Guess cte for mewencode executed later.
       ;; xxx how about mew-encode-escape-region?
       (if cte
@@ -170,8 +326,7 @@ to multipart, and so on."
       (if (null encopts)
 	  ()
 	;; We MUST save text since charset MAY change.
-	(mew-frwlet
-	 mew-cs-noconv (mew-charset-to-cs charset)
+	(mew-frwlet mew-cs-dummy (mew-charset-to-cs charset)
 	 (setq outfile (mew-make-temp-name))
 	 ;; NEVER use call-process-region for privary reasons
 	 (write-region (point) (point-max) outfile nil 'no-msg))
@@ -184,8 +339,8 @@ to multipart, and so on."
   ;; buffered is t if called make-single
   (let* ((file (expand-file-name (mew-syntax-get-file syntax) path))
 	 (ctl (mew-syntax-get-ct syntax))
-	 (ct (capitalize (car ctl))) ;; just in case
-	 (ctparams (cdr ctl))
+	 (ct (mew-syntax-get-value ctl 'cap))
+	 (ctparams (mew-syntax-get-params ctl))
 	 (cte (mew-syntax-get-cte syntax))
 	 (cd (mew-syntax-get-cd syntax))
 	 (cdpl (mew-syntax-get-cdp syntax))
@@ -195,7 +350,7 @@ to multipart, and so on."
 	 (beg (point))
 	 (text-sw mew-prog-mime-encode-text-switch)
 	 (binary-sw mew-prog-mime-encode-switch)
-	 charset fcce encopts)
+	 charset fcce encopts bodybeg)
     (if (null (string-match "Text/" ct))
 	;; No need to guess charset.
 	;; The text based object is encoded by mewencode replacing LF 
@@ -211,40 +366,37 @@ to multipart, and so on."
       (setq file (nth 0 fcce) charset (nth 1 fcce) 
 	    cte (nth 2 fcce) encopts (nth 3 fcce))
       ;; file is nil if buffered
-      (setq ctparams (list (format "charset=%s" charset))))
+      (setq ctparams (list (list "charset" charset))))
     ;; Insert ct and its ctparams. This is flexible enough for msg/extb
-    (mew-header-insert-here mew-ct: ct 'noret)
-    (mew-header-insert-param-here ctparams)
-    (mew-header-insert "\n")
+    (mew-header-insert mew-ct: ct ctparams)
     ;; Insert cte.
-    (mew-header-insert-here mew-cte: cte) ;; cte is never nil here
+    (mew-header-insert mew-cte: cte) ;; cte is never nil here
     ;; Insert cd.
-    (if cd
-	(progn
-	  (mew-header-insert-here mew-cd: t 'noret)
-	  (mew-header-encode-text cd)
-	  (mew-header-insert "\n")))
-    (if cdp
-	(progn
-	  (mew-header-insert-here mew-cdp: cdp 'noret)
-	  (mew-header-insert-param-here cdpparams)
-	  (mew-header-insert "\n")))
+    (and cd (mew-header-insert mew-cd: cd))
+    (and cdp (mew-header-insert mew-cdp: cdp cdpparams))
     ;; Terminate content-header.
     (insert "\n")
     ;; header "\n" (cur) [text]
+    (setq bodybeg (point))
     (if encopts
 	;; Binary objects matches here.
 	;; If called charset-magic and encopts is not nil, file is not nil.
 	;; Encoded file is always in us-ascii domain. So just insert it.
-	(mew-piolet mew-cs-autoconv mew-cs-noconv
+	(mew-piolet mew-cs-autoconv mew-cs-text-for-write
 	 (apply (function call-process) mew-prog-mime-encode 
 		file t nil encopts))
-      (if file ;; not buffered	 
+      (if (and (stringp file) (file-readable-p file))
+	  ;; not buffered	 
 	  ;; Probably 7bit text based object like phantom body of 
 	  ;; message/external-body. 
 	  (insert-file-contents file)))
+    (if (and (mew-case-equal ct mew-ct-msg) mew-field-delete-for-forwarding)
+	(save-restriction
+	  (narrow-to-region bodybeg (point-max))
+	  (mew-header-delete-lines mew-field-delete-common)
+	  (mew-header-delete-lines mew-field-delete-for-forwarding)))
     ;; Go to RFC1847 privacy services.
-    ;; RFC1847 and services using it such as RFC1848 MOSS and RFC2015 PGP/MIME
+    ;; RFC1847 and services using it such as RFC2015 PGP/MIME
     ;; allows 8bit/binary MIME canonical form(i.e. 7bit content-header and 
     ;; 8bit/binary content-body) for encryption (not for signing). 
     ;; But Mew restricts an object to be ecnrypted(or, of course, signed) 
@@ -281,16 +433,14 @@ to multipart, and so on."
     (goto-char (point-max))))
 
 (defun mew-draft-make-multi ()
-  (mew-draft-make-backup)
   ;; delete delimiter
-  (goto-char (mew-header-end))
+  (goto-char (mew-header-end)) ;; due to illegal null lines in the header
   (forward-line)
-  (let* ((beg (point))
-	 (syntax mew-encode-syntax) 
-	 ;; use local variable to prevent side effects when undo
-	 (cpath (mew-expand-folder (mew-draft-to-mime (buffer-name))))
-	 (coverfile (expand-file-name mew-draft-coverpage cpath))
-	 (path (mew-expand-folder mew-draft-mime-folder)))
+  (let ((beg (point))
+	(syntax mew-encode-syntax) 
+	;; use local variable to prevent side effects when undo
+	(coverfile (expand-file-name mew-draft-coverpage (mew-attachdir)))
+	(path (mew-expand-folder mew-draft-mime-folder)))
     ;; just after the header
     ;; see if cover page is empty or not
     (while (and (looking-at "^$") (not (eobp)))
@@ -299,8 +449,7 @@ to multipart, and so on."
 	;; A cover page exists, so save it.
 	;; On Mule, from internal to mew-cs-draft.
 	;; This code may be overhead but makes code simple.
-	(mew-frwlet
-	 mew-cs-noconv mew-cs-draft
+	(mew-frwlet mew-cs-dummy mew-cs-draft
 	 (write-region beg (point-max) coverfile nil 'no-msg))
       ;; A cover page doesn't exist
       ;; Remove the cover page entry from the syntax
@@ -318,21 +467,16 @@ to multipart, and so on."
 	   (if (> depth 0) (format "BOUNDARY%s" (int-to-string depth)))))
 	 (fullname (expand-file-name (mew-syntax-get-file syntax) path))
 	 (ctl (mew-syntax-get-ct syntax))
-	 (ct (capitalize (car ctl))) ;; just in case
+	 (ct (mew-syntax-get-value ctl 'cap))
 	 (cte (mew-syntax-get-cte syntax))
 	 (cd (mew-syntax-get-cd syntax))
 	 (privacy (mew-syntax-get-privacy syntax))
 	 (len (length syntax))
 	 (cnt mew-syntax-magic)
 	 (beg (point)))
-    (mew-header-insert-here 
-     mew-ct: (concat ct ";\n\tboundary=\"" boundary "\""))
-    (mew-header-insert-here mew-cte: (or cte mew-7bit))
-    (if cd
-	(progn
-	  (mew-header-insert-here mew-cd: t 'noret)
-	  (mew-header-encode-text cd)
-	  (mew-header-insert "\n")))
+    (mew-header-insert mew-ct: ct (list (list "boundary" boundary)))
+    (mew-header-insert mew-cte: (or cte mew-7bit))
+    (and cd (mew-header-insert mew-cd: cd))
     (while (< cnt len)
       (insert (concat "\n--" boundary "\n"))
       (if (mew-syntax-multipart-p (aref syntax cnt))
@@ -377,9 +521,7 @@ to multipart, and so on."
 	 (switch mew-encode-multipart-encrypted-switch) ;; save length
 	 (func (cdr (mew-assoc-case-equal protocol switch 0)))
 	file1 file2 file3 cte2 cte3 fc error)
-    (setq decrypters 
-	  (cons mew-mail-address
-		(mew-addrstr-canonicalize-address-list decrypters)))
+    (setq decrypters (cons mew-inherit-signer decrypters))
     ;; Write the part converting line breaks.
     (setq file1 (mew-save-transfer-form (point-min) (point-max) nil))
     ;; The narrowed region stores nothing
@@ -397,14 +539,12 @@ to multipart, and so on."
 	  (if (file-exists-p file1) (delete-file file1))
 	  (if (file-exists-p file2) (delete-file file2))
 	  (if (file-exists-p file3) (delete-file file3))
-	  (error (concat error " Type "
-			 (substitute-command-keys
-			  "\\<mew-draft-mode-map>\\[mew-draft-undo]")
-			 " to get the draft back.")))
+	  (mew-draft-undo)
+	  (setq mew-draft-privacy-error t)
+	  (error error))
       ;; Create multipart content-header
-      (mew-header-insert-here mew-ct: ct 'noret)
-      (mew-header-insert (format ";\n\tprotocol=\"%s\"" protocol))
-      (mew-header-insert (format ";\n\tboundary=\"%s\"\n" boundary)) ;; \n doubled
+      (mew-header-insert mew-ct: ct (list (list "protocol" protocol)
+					       (list "boundary" boundary)))
       (insert (format "\n--%s\n" boundary))
       ;; Insert control keys
       (mew-encode-singlepart 
@@ -440,18 +580,15 @@ to multipart, and so on."
 	(progn
 	  (if (file-exists-p file1) (delete-file file1))
 	  (if (file-exists-p file2) (delete-file file2))
-	  (error (concat error " Type "
-			 (substitute-command-keys
-			  "\\<mew-draft-mode-map>\\[mew-draft-undo]")
-			 " to get the draft back.")))
+	  (mew-draft-undo)
+	  (setq mew-draft-privacy-error t)
+	  (error error))
       (goto-char (point-min))
       ;; Before the signed part
       ;; Create multipart content-header
-      (mew-header-insert-here mew-ct: ct 'noret)
-      (mew-header-insert (format ";\n\tprotocol=\"%s\"" protocol))
-      (mew-header-insert (format ";\n\tmicalg=\"%s\"" micalg))
-      (mew-header-insert (format ";\n\tboundary=\"%s\"\n" boundary)) 
-      ;; \n doubled
+      (mew-header-insert mew-ct: ct (list (list "protocol" protocol)
+					       (list "micalg" micalg)
+					       (list "boundary" boundary)))
       (insert (format "\n--%s\n" boundary))
       (goto-char (point-max))
       ;; After the sigend part
@@ -478,7 +615,7 @@ to multipart, and so on."
       (while (search-forward "\n" nil t) (replace-match "\r\n" nil t))
       (setq send (point-max)))
     (setq file (mew-make-temp-name))
-    (mew-frwlet mew-cs-noconv mew-cs-7bit-crlf
+    (mew-frwlet mew-cs-dummy mew-cs-7bit-crlf
       (write-region sbeg send file nil 'no-msg))
     (if retain
 	(mew-erase-buffer)
@@ -504,7 +641,7 @@ to multipart, and so on."
     (cond 
      ((string-match "iso-8859" charset) t)
      ((string-match "iso-2022" charset) nil)
-     ((string-match "us-ascii" charset) nil)
+     ((string-match mew-us-ascii charset) nil)
      (t t) ;; just in case
      )))
 
@@ -519,7 +656,7 @@ to multipart, and so on."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996, 1997, 1998 Mew developing team.
+;; Copyright (C) 1996, 1997, 1998, 1999 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
