@@ -2,11 +2,11 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Nov 13, 1996
-;; Revised: Aug 24, 1998
+;; Revised: Aug 30, 1999
 
 ;;; Code:
 
-(defconst mew-ext-version "mew-ext.el version 0.20")
+(defconst mew-ext-version "mew-ext.el version 0.21")
 
 (require 'mew)
 (eval-when-compile
@@ -129,15 +129,15 @@ w3 is used.")
   (interactive)
   (if (not (mew-attach-not-line012-1))
       (message "Can't insert external-body here")
-    (let* ((nums (mew-attach-nums))
+    (let* ((nums (mew-syntax-nums))
 	   (subdir (mew-attach-expand-path mew-encode-syntax nums))
-	   (mimedir (mew-expand-folder (mew-draft-to-mime (buffer-name))))
+	   (attachdir (mew-attachdir))
 	   file filepath ct)
-      ;; mimedir / {subdir/} dir
+      ;; attachdir / {subdir/} dir
       (if (not (equal subdir "")) 
-	  (setq mimedir (expand-file-name subdir mimedir)))
-      ;; mimedir / file
-      (setq filepath (mew-random-filename mimedir mew-ext-suffix))
+	  (setq attachdir (expand-file-name subdir attachdir)))
+      ;; attachdir / file
+      (setq filepath (mew-random-filename attachdir mew-ext-suffix))
       (if (file-exists-p filepath)
 	  (message "Could not make a file for external-body, sorry.")
 	(setq file (file-name-nondirectory filepath))
@@ -176,15 +176,15 @@ w3 is used.")
 	(setq ct mew-ct-apo))
        (t 
 	(setq name (file-name-nondirectory
-		    (mew-chop (mew-syntax-get-member ret "name"))))
+		    (mew-chop (mew-syntax-get-param ret "name"))))
 	;; name is quoted
 	(if (equal name "")
 	    (setq ct mew-ct-apo)
 	  (setq ct (capitalize (mew-attr-get-ct (mew-attr-by-file name)))))))
       (setq ct (mew-input-type "Type for %s (%s): " name ct
 			       mew-mime-content-type-list))
-      (mew-header-insert-here mew-ct: ct)
-      (mew-header-insert-here mew-cid: (mew-create-content-id))
+      (mew-header-insert mew-ct: ct)
+      (mew-header-insert mew-cid: (mew-create-content-id))
       (insert "\n")
       (if (not (string= access-type "mail-server"))
 	  () ;; message-body is not necessary
@@ -193,7 +193,7 @@ w3 is used.")
 	(insert "\n"))
       (write-file filename))
     (kill-buffer buf)
-    (cons mew-ct-ext (cons (format "access-type=\"%s\"" access-type) ret))))
+    (cons mew-ct-ext (cons (list "access-type" access-type) ret))))
 
 (defun mew-ext-encode-ftp ()
   ;; "name" "site" "directory" "mode"
@@ -205,7 +205,7 @@ w3 is used.")
 		       nil
 		       (car mew-ext-ftp-server-list)))
 	mew-ext-user path dir file ret)
-    (setq ret (list (format "site=\"%s\"" mew-ext-host)))
+    (setq ret (list (list "site" mew-ext-host)))
     (setq mew-ext-user (read-string (format "User name at %s: " mew-ext-host)
 				(user-login-name)))
     (setq path (mew-input-rfile "Filename :"))
@@ -215,10 +215,10 @@ w3 is used.")
 	(setq dir (mew-ext-expand-dir mew-ext-host mew-ext-user dir)))
     (cond
      (dir
-      (setq ret (cons (format "directory=\"%s\"" dir) ret))
-      (setq ret (cons (format "name=\"%s\"" file) ret)))
+      (setq ret (cons (list "directory" dir) ret))
+      (setq ret (cons (list "name" file) ret)))
      (t
-      (setq ret (cons (format "name=\"%s\"" file) ret))))
+      (setq ret (cons (list "name" file) ret))))
     ret))
 
 (defun mew-ext-encode-tftp ()
@@ -236,7 +236,7 @@ w3 is used.")
 		       nil
 		       (car mew-ext-ftp-server-list)))
 	path dir file ret)
-    (setq ret (list (format "site=\"%s\"" mew-ext-host)))
+    (setq ret (list (list "site" mew-ext-host)))
     (setq path (mew-input-rfile "Filename :"))
     (setq file (file-name-nondirectory path))
     (setq dir (file-name-directory path))
@@ -244,29 +244,29 @@ w3 is used.")
 	(setq dir (mew-ext-expand-dir mew-ext-host mew-ext-user dir)))
     (cond
      (dir
-      (setq ret (cons (format "directory=\"%s\"" dir) ret))
-      (setq ret (cons (format "name=\"%s\"" file) ret)))
+      (setq ret (cons (list "directory" dir) ret))
+      (setq ret (cons (list "name" file) ret)))
      (t
-      (setq ret (cons (format "name=\"%s\"" file) ret))))
+      (setq ret (cons (list "name" file) ret))))
     ret))
 
 (defun mew-ext-encode-local-file ()
   ;; "name" "site"
-  (let ((file (mew-input-file-name "File name: ")))
-    (list (format "name=\"%s\"" (expand-file-name file)))))
+  (let ((file (mew-draft-input-file-name "File name: ")))
+    (list (list "name" (expand-file-name file)))))
 
 (defun mew-ext-encode-mail-server ()
   ;; "server" "subject"
   (let (server subject)
-    (setq server (mew-input-address "Server address:"))
+    (setq server (car (mew-input-address "Server address: ")))
     (setq subject (read-string (concat mew-subj: " ")))
-    (list (format "server=\"%s\"" server)
-	  (format "subject=\"%s\"" subject))))
+    (list (list "server" server)
+	  (list "subject" subject))))
    
 (defun mew-ext-encode-url ()
   ;; "url"
   (let ((url (read-string "URL: ")))
-    (list (format "url=\"%s\"" url))))
+    (list (list "url" url))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -279,189 +279,184 @@ w3 is used.")
 
 (defun mew-mime-external-body (begin end params &optional execute)
   ;; message-buffer
-  (let* ((access-type (mew-syntax-get-member params "access-type"))
+  (let* ((access-type (mew-syntax-get-param params "access-type"))
 	 (program (cdr (mew-assoc-match access-type mew-ext-switch 0))))
     ;; xxx expire
     (if (and (symbolp program) (fboundp program))
 	(funcall program begin end params execute))))
 
 (defun mew-ext-ftp (begin end params execute)
-  (let* ((site (mew-syntax-get-member params "site"))
-	 (directory (mew-syntax-get-member params "directory"))
-	 (name (mew-syntax-get-member params "name"))
-	 (size (mew-syntax-get-member params "size"))
-	 (getit t) (username "") (buffer-read-only nil)
-	 filepath localfile lfname remotefile)
-    (if directory
-	(setq filepath (concat (file-name-as-directory directory) name))
-      (setq filepath name))
-    (if (and filepath (string-match "^[A-Za-z]:/.+" filepath)) ;; drive letter
-	(setq filepath (substring filepath 2 (match-end 0))))
-    (erase-buffer)
-    (insert " ####### ####### ######  \n"
-	    " #          #    #     # \n"
-	    " #          #    #     # \n"
-	    " #####      #    ######  \n"
-	    " #          #    #       \n"
-	    " #          #    #       \n"
-	    " #          #    #       \n"
-            "\n\n")
-    (insert "You can get the message content by FTP\n\n")
-    (mew-insert "SITE:\t%s\n" site)
-    (mew-insert "FILE:\t%s\n" filepath)
-    (mew-insert "SIZE:\t%s bytes\n" size)
-    (if (null execute)
-	(insert "\nTo get this file, type "
-		(substitute-command-keys
-		 "\\<mew-summary-mode-map>\\[mew-summary-execute-external]."))
-      (setq username (read-string (format "User name at %s: " site)
-				  (user-login-name)))
-      (setq remotefile (format "/%s@%s:%s" username site filepath))
-      (setq localfile (mew-input-file-name 
-		       "Save to: "
-		       (format "%s%s" default-directory name)))
-      (setq lfname (file-name-nondirectory localfile))
-      (if (file-exists-p localfile)
-	  (if (y-or-n-p (format "%s exists. Overwrite? " lfname))
-	      (delete-file localfile)
-	    (setq getit nil)
-	    (message "The file wasn't retrieved")))
-      (if getit (mew-ext-copy-file-internal remotefile localfile nil)))))
+  (mew-elet
+   (let* ((site (mew-syntax-get-param params "site"))
+	  (directory (mew-syntax-get-param params "directory"))
+	  (name (mew-syntax-get-param params "name"))
+	  (size (mew-syntax-get-param params "size"))
+	  (getit t) (username "")
+	  filepath localfile lfname remotefile)
+     (if directory
+	 (setq filepath (concat (file-name-as-directory directory) name))
+       (setq filepath name))
+     (if (and filepath (string-match "^[A-Za-z]:/.+" filepath)) ;; drive letter
+	 (setq filepath (substring filepath 2 (match-end 0))))
+     (mew-erase-buffer)
+     (insert " ####### ####### ######  \n"
+	     " #          #    #     # \n"
+	     " #          #    #     # \n"
+	     " #####      #    ######  \n"
+	     " #          #    #       \n"
+	     " #          #    #       \n"
+	     " #          #    #       \n"
+	     "\n\n")
+     (insert "You can get the message content by FTP\n\n")
+     (mew-insert "SITE:\t%s\n" site)
+     (mew-insert "FILE:\t%s\n" filepath)
+     (mew-insert "SIZE:\t%s bytes\n" size)
+     (if (null execute)
+	 (insert "\nTo get this file, type "
+		 (substitute-command-keys
+		  "'\\<mew-summary-mode-map>\\[mew-summary-execute-external]'."))
+       (setq username (read-string (format "User name at %s: " site)
+				   (user-login-name)))
+       (setq remotefile (format "/%s@%s:%s" username site filepath))
+       (setq localfile (mew-summary-input-file-name "Save to: " name))
+       (setq lfname (file-name-nondirectory localfile))
+       (if (file-exists-p localfile)
+	   (if (y-or-n-p (format "%s exists. Overwrite? " lfname))
+	       (delete-file localfile)
+	     (setq getit nil)
+	     (message "The file wasn't retrieved")))
+       (if getit (mew-ext-copy-file-internal remotefile localfile nil))))))
 
 (defun mew-ext-tftp (begin end params execute)
   (message "access-type TFTP is not supported yet"))
 
 (defun mew-ext-anon-ftp (begin end params execute)
-  (let* ((site (mew-syntax-get-member params "site"))
-	 (directory (mew-syntax-get-member params "directory"))
-	 (name (mew-syntax-get-member params "name"))
-	 (size (mew-syntax-get-member params "size"))
-	 (getit t) (buffer-read-only nil)
-	 filepath localfile lfname remotefile url)
-    (if directory
-	(setq filepath (concat (file-name-as-directory directory) name))
-      (setq filepath name))
-    (if (and filepath (string-match "^[A-Za-z]:/.+" filepath)) ;; drive letter
-	(setq filepath (substring filepath 2 (match-end 0))))
-    (erase-buffer)
-    (insert " Anonymous \n"
-            " ####### ####### ######  \n"
-	    " #          #    #     # \n"
-	    " #          #    #     # \n"
-	    " #####      #    ######  \n"
-	    " #          #    #       \n"
-	    " #          #    #       \n"
-	    " #          #    #       \n"
-            "\n\n")
-    (insert "You can get the message content by FTP\n\n")
-    (mew-insert "SITE:\t%s\n" site)
-    (mew-insert "FILE:\t%s\n" filepath)
-    (mew-insert "SIZE:\t%s bytes\n" size)
-    (if (null execute)
-	(insert "\nTo get this file, type "
-		(substitute-command-keys
-		 "\\<mew-summary-mode-map>\\[mew-summary-execute-external]."))
-      (setq remotefile (format "/%s@%s:%s" "anonymous" site filepath))
-      (if (string-match "^[/\:]" filepath)
-	  (setq url (format "ftp://%s%s" site filepath))
-	(setq url (format "ftp://%s/%s" site filepath)))
-      (setq localfile (mew-input-file-name 
-		       "Save to: "
-		       (format "%s%s" default-directory name)))
-      (setq lfname (file-name-nondirectory localfile))
-      (if (file-exists-p localfile)
-	  (if (y-or-n-p (format "%s exists. Overwrite? " lfname))
-	      (delete-file localfile)
-	    (setq getit nil)
-	    (message "The file wasn't retrieved"))
-	)
-      (if (not getit)
-	  ()
-	(cond
-	 ((eq mew-ext-anon-ftp-method 'ftp)
-	  (mew-ext-copy-file-internal remotefile localfile mew-mail-address))
-	 ((eq mew-ext-anon-ftp-method 'http)
-	  (require 'w3)
-	  (w3-fetch url)))))))
+  (mew-elet
+   (let* ((site (mew-syntax-get-param params "site"))
+	  (directory (mew-syntax-get-param params "directory"))
+	  (name (mew-syntax-get-param params "name"))
+	  (size (mew-syntax-get-param params "size"))
+	  (getit t)
+	  filepath localfile lfname remotefile url)
+     (if directory
+	 (setq filepath (concat (file-name-as-directory directory) name))
+       (setq filepath name))
+     (if (and filepath (string-match "^[A-Za-z]:/.+" filepath)) ;; drive letter
+	 (setq filepath (substring filepath 2 (match-end 0))))
+     (mew-erase-buffer)
+     (insert " Anonymous \n"
+	     " ####### ####### ######  \n"
+	     " #          #    #     # \n"
+	     " #          #    #     # \n"
+	     " #####      #    ######  \n"
+	     " #          #    #       \n"
+	     " #          #    #       \n"
+	     " #          #    #       \n"
+	     "\n\n")
+     (insert "You can get the message content by FTP\n\n")
+     (mew-insert "SITE:\t%s\n" site)
+     (mew-insert "FILE:\t%s\n" filepath)
+     (mew-insert "SIZE:\t%s bytes\n" size)
+     (if (null execute)
+	 (insert "\nTo get this file, type "
+		 (substitute-command-keys
+		  "'\\<mew-summary-mode-map>\\[mew-summary-execute-external]'."))
+       (setq remotefile (format "/%s@%s:%s" "anonymous" site filepath))
+       (if (string-match "^[/\:]" filepath)
+	   (setq url (format "ftp://%s%s" site filepath))
+	 (setq url (format "ftp://%s/%s" site filepath)))
+       (setq localfile (mew-summary-input-file-name "Save to: " name))
+       (setq lfname (file-name-nondirectory localfile))
+       (if (file-exists-p localfile)
+	   (if (y-or-n-p (format "%s exists. Overwrite? " lfname))
+	       (delete-file localfile)
+	     (setq getit nil)
+	     (message "The file wasn't retrieved")))
+       (if (not getit)
+	   ()
+	 (cond
+	  ((eq mew-ext-anon-ftp-method 'ftp)
+	   (mew-ext-copy-file-internal remotefile localfile mew-mail-address))
+	  ((eq mew-ext-anon-ftp-method 'http)
+	   (require 'w3)
+	   (w3-fetch url))))))))
 
 (defun mew-ext-mail-server (begin end params execute)
-  (let ((server (mew-syntax-get-member params "server"))
-	(subject (mew-syntax-get-member params "subject"))
-	(size (mew-syntax-get-member params "size"))
-	(start nil)
-	(buffer-read-only nil))
-
-    (erase-buffer)
-    (insert " #     #    #      ###   #\n"
-            " ##   ##   # #      #    #\n"
-            " # # # #  #   #     #    #\n"
-            " #  #  # #     #    #    #\n"
-            " #     # #######    #    #\n"
-            " #     # #     #    #    #\n"
-            " #     # #     #   ###   #######\n"
-            "\n\n")
-    (insert "You can get the message by e-mail\n\n")
-    (mew-insert "SERVER:\t\t%s\n" server)
-    (mew-insert "SIZE:\t%s bytes\n" size)
-    (save-excursion
-      (set-buffer (mew-current-get 'cache))
-      (save-restriction
-	(narrow-to-region begin end)
-	(goto-char (point-min))
-	;; find a phantom body (in RFC1521)
-	(re-search-forward "^$" nil t)
-	(forward-line)
-	(setq start (point))
-	))
-    ;; pickd up source from 'mew-send
-    (if (null execute)
-	(insert "\nTo send this mail, type "
-		(substitute-command-keys
-		 "\\<mew-summary-mode-map>\\[mew-summary-execute-external]."))
-      (mew-summary-send server nil subject)
-      (goto-char (point-max))
-      (insert-buffer-substring (mew-current-get 'cache) start end)
-      (mew-draft-make-message)
-      (if (y-or-n-p "Send this message? ")
-	  (mew-draft-real-send-letter)))))
+  (mew-elet
+   (let ((server (mew-syntax-get-param params "server"))
+	 (subject (mew-syntax-get-param params "subject"))
+	 (size (mew-syntax-get-param params "size"))
+	 (start nil))
+     (mew-erase-buffer)
+     (insert " #     #    #      ###   #\n"
+	     " ##   ##   # #      #    #\n"
+	     " # # # #  #   #     #    #\n"
+	     " #  #  # #     #    #    #\n"
+	     " #     # #######    #    #\n"
+	     " #     # #     #    #    #\n"
+	     " #     # #     #   ###   #######\n"
+	     "\n\n")
+     (insert "You can get the message by e-mail\n\n")
+     (mew-insert "SERVER:\t\t%s\n" server)
+     (mew-insert "SIZE:\t%s bytes\n" size)
+     (save-excursion
+       (set-buffer (mew-current-get 'cache))
+       (save-restriction
+	 (narrow-to-region begin end)
+	 (goto-char (point-min))
+	 ;; find a phantom body (in RFC1521)
+	 (re-search-forward "^$" nil t)
+	 (forward-line)
+	 (setq start (point))))
+     ;; pickd up source from 'mew-send
+     (if (null execute)
+	 (insert "\nTo send this mail, type "
+		 (substitute-command-keys
+		  "'\\<mew-summary-mode-map>\\[mew-summary-execute-external]'."))
+       (mew-summary-send server nil subject)
+       (goto-char (point-max))
+       (insert-buffer-substring (mew-current-get 'cache) start end)
+       (mew-draft-make-message)
+       (if (y-or-n-p "Send this message? ")
+	   (mew-draft-real-send-letter))))))
 
 (defun mew-ext-url (begin end params execute)
-  (let ((url (mew-syntax-get-member params "url"))
-	(size (mew-syntax-get-member params "size"))
-	(process-connection-type mew-connection-type1)
-	(buffer-read-only nil))
-    (erase-buffer)
-    (insert "#     # ######  #\n"
-	    "#     # #     # #\n"
-	    "#     # #     # #\n"
-	    "#     # ######  #\n"
-	    "#     # #   #   #\n"
-	    "#     # #    #  #\n"
-	    " #####  #     # #######\n"
-	    "\n\n")
-    (mew-insert "URL:\t\t%s\n" url)
-    (mew-insert "SIZE:\t%s bytes\n" size)
-    (insert (format "Browser:\t%s\n"
-		    (cond ((and (symbolp mew-ext-prog-url)
-				(fboundp mew-ext-prog-url))
-			   (symbol-name mew-ext-prog-url))
-			  ((stringp mew-ext-prog-url) mew-ext-prog-url)
-			  (t "none"))))
-    (if (null execute)
-	(insert "\nTo show this URL, type "
-		(substitute-command-keys
-		 "\\<mew-summary-mode-map>\\[mew-summary-execute-external]."))
-      (cond
-       ((and (symbolp mew-ext-prog-url) (fboundp mew-ext-prog-url))
-	(funcall mew-ext-prog-url url))
-       ((equal mew-ext-prog-url "w3")
-	(require 'w3)
-	(w3-fetch-other-frame url))
-       (t
-	(apply (function start-process)
-	       (format "*mew %s*" mew-ext-prog-url)
-	       mew-buffer-tmp mew-ext-prog-url 
-	       (append mew-ext-prog-url-args (list url))))))))
+  (mew-elet
+   (let ((url (mew-syntax-get-param params "url"))
+	 (size (mew-syntax-get-param params "size"))
+	 (process-connection-type mew-connection-type1))
+     (mew-erase-buffer)
+     (insert "#     # ######  #\n"
+	     "#     # #     # #\n"
+	     "#     # #     # #\n"
+	     "#     # ######  #\n"
+	     "#     # #   #   #\n"
+	     "#     # #    #  #\n"
+	     " #####  #     # #######\n"
+	     "\n\n")
+     (mew-insert "URL:\t\t%s\n" url)
+     (mew-insert "SIZE:\t%s bytes\n" size)
+     (insert (format "Browser:\t%s\n"
+		     (cond ((and (symbolp mew-ext-prog-url)
+				 (fboundp mew-ext-prog-url))
+			    (symbol-name mew-ext-prog-url))
+			   ((stringp mew-ext-prog-url) mew-ext-prog-url)
+			   (t "none"))))
+     (if (null execute)
+	 (insert "\nTo show this URL, type "
+		 (substitute-command-keys
+		  "'\\<mew-summary-mode-map>\\[mew-summary-execute-external]'."))
+       (cond
+	((and (symbolp mew-ext-prog-url) (fboundp mew-ext-prog-url))
+	 (funcall mew-ext-prog-url url))
+	((equal mew-ext-prog-url "w3")
+	 (require 'w3)
+	 (w3-fetch-other-frame url))
+	(t
+	 (apply (function start-process)
+		(format "*mew %s*" mew-ext-prog-url)
+		mew-buffer-tmp mew-ext-prog-url 
+		(append mew-ext-prog-url-args (list url)))))))))
 
 ;;
 ;; include
@@ -469,7 +464,7 @@ w3 is used.")
 
 (defun mew-ext-include-local-file (params)
   (mew-flet
-   (let* ((file (mew-syntax-get-member params "name")))
+   (let* ((file (mew-syntax-get-param params "name")))
      (if (file-exists-p file)
 	 (insert-file-contents file)))))
 
@@ -477,7 +472,7 @@ w3 is used.")
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996, 1997, 1998 Mew developing team.
+;; Copyright (C) 1996, 1997, 1998, 1999 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
