@@ -2,11 +2,11 @@
 
 ;; Author:  Yoshinari NOMURA <nom@csce.kyushu-u.ac.jp>
 ;; Created: Nov 29, 1994
-;; Revised: Aug 24, 1998
+;; Revised: Aug 30, 1999
 
 ;;; Code:
 
-(defconst mew-fib-version "mew-fib.el version 0.10")
+(defconst mew-fib-version "mew-fib.el version 0.12")
 
 (require 'mew-func)
 
@@ -38,13 +38,13 @@
 	(setq item    (buffer-substring (match-beginning 1) (match-end 1))
 	      tmp-val (buffer-substring (match-beginning 2) (match-end 2))
 	      val (if (string= tmp-val "") val tmp-val)
-	      ret (append ret (mapcar
-			       (lambda (arg) (cons (downcase arg) val))
-			       (mew-fib-split item)))))
+	      ret (append ret (mapcar (function (lambda (arg)
+						  (cons (downcase arg) val)))
+				      (mew-fib-split item)))))
       ret)))
 
 (defun mew-fib-fill-default ()
-  "Fill |>item<| from .mew-fib."
+  "Fill |>item<| acording to the information from .mew-fib"
   (interactive)
   (save-excursion
     (let (begin end str)
@@ -60,7 +60,7 @@
     )))
 
 (defun mew-fib-delete-frame ()
-  "Delete all quotations, i.e. |> <|."
+  "Delete all quotations, '|>' and '<|'. This is the finishing stroke."
   (interactive)
   (save-excursion
     (goto-char (point-min))
@@ -69,12 +69,12 @@
       )))
 
 (defun mew-fib-flush-input ()
-  "Flush input from .mew-fib."
+  "Flush the fib item on the current point."
   (interactive)
   (save-excursion
     (let ((ptr (point)))
-      (goto-char (point-min))
-      (if (and (looking-at "^.*|>\\([^<]+\\)<|")
+      (if (and (search-backward "|>" nil t)
+	       (looking-at "|>\\([^<]+\\)<|")
 	       (>= ptr (match-beginning 1))
 	       (<= ptr (match-end 1)))
 	  (delete-region (match-beginning 1)
@@ -100,7 +100,7 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1994, 1995, 1996, 1997, 1998 Mew developing team.
+;; Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
