@@ -2,64 +2,63 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Oct  2, 1996
-;; Revised: Aug 24, 1998
+;; Revised: Aug 31, 1999
 
 ;;; Code:
 
-(defconst mew-header-version "mew-header.el version 0.13")
+(defconst mew-header-version "mew-header.el version 0.15")
 
 (require 'mew)
 
+(defvar mew-anonymous-recipients ":;")
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Header functions
 ;;;
 
 (defmacro mew-header-goto-next ()
-  (` (while (looking-at mew-lwsp) (forward-line))))
+  '(while (looking-at mew-lwsp)
+     (forward-line)))
 
 ;; see also mew-header-end
 (defmacro mew-header-goto-end ()
-  (` (progn
-       (goto-char (point-min))
-       (if (re-search-forward mew-eoh nil t)
-	   (beginning-of-line)
-	 (goto-char (point-max))
-	 (if (eolp) (insert "\n"))))))
+  '(progn
+     (goto-char (point-min))
+     (if (re-search-forward mew-eoh nil t)
+	 (beginning-of-line)
+       (goto-char (point-max))
+       (if (eolp) (insert "\n")))))
 
 (defmacro mew-header-goto-body ()
-  (` (progn
-       (mew-header-goto-end)
-       (forward-line))))
+  '(progn
+     (mew-header-goto-end)
+     (forward-line)))
 
 (defun mew-header-get-value (field)
   "currently, when no match, it returns nil."
   ;; maybe called in narrowed region.
   ;; we can't widen for citation.
   (let ((case-fold-search t)
-	(start nil)
-	(match nil)
-	(ret nil))
+	(regex (format "^%s[ \t]*" field))
+	start match ret)
     (save-excursion
       (mew-header-goto-end)
       (save-restriction
 	(narrow-to-region (point-min) (point))
-	(if (>= (count-lines (point-min) (point-max)) 
-		mew-header-max-length)
-	    () ;; xxx error?
-	  (goto-char (point-min))
-	  (catch 'only-one
-	    (while (re-search-forward (format "^%s[ \t]*" field) nil t)
-	      (setq start (match-end 0))
-	      (forward-line)
-	      (mew-header-goto-next)
-	      (setq match (mew-buffer-substring start (1- (point))))
-	      (if (string= "" match)
-		  ()
-		(if ret
-		    (setq ret (concat ret "," match))
-		  (setq ret match)
-		  (if (equal field mew-from:) (throw 'only-one nil)))))))))
+	(goto-char (point-min))
+	(catch 'only-one
+	  (while (re-search-forward regex nil t)
+	    (setq start (match-end 0))
+	    (forward-line)
+	    (mew-header-goto-next)
+	    (setq match (mew-buffer-substring start (1- (point))))
+	    (if (string= "" match)
+		()
+	      (if ret
+		  (setq ret (concat ret "," match))
+		(setq ret match)
+		(if (equal field mew-from:) (throw 'only-one nil))))))))
     ret))
 
 (fset 'mew-header-existp (symbol-function 'mew-header-get-value))
@@ -68,49 +67,76 @@
   (concat "^\\(" (mapconcat (function identity) fields "\\|") "\\)"))
 
 (defun mew-header-delete-lines (fields)
-  (let ((case-fold-search t)
-	(regex (mew-make-field-regex fields))
-	start)
-    (mew-header-goto-end)
-    (save-restriction
-      (narrow-to-region (point-min) (point))
-      (goto-char (point-min))
-      (while (re-search-forward regex nil t)
-	(setq start (match-beginning 0))
-	(forward-line)
-	(mew-header-goto-next)
-	(delete-region start (point))))))
+  (if (null fields)
+      ()
+    (let ((case-fold-search t)
+	  (regex (mew-make-field-regex fields))
+	  start)
+      (mew-header-goto-end)
+      (save-restriction
+	(narrow-to-region (point-min) (point))
+	(goto-char (point-min))
+	(while (re-search-forward regex nil t)
+	  (setq start (match-beginning 0))
+	  (forward-line)
+	  (mew-header-goto-next)
+	  (delete-region start (point)))))))
+
+(defun mew-header-replace-lines (fields prefix)
+  (if (null fields)
+      ()
+    (let ((case-fold-search t)
+	  (regex (mew-make-field-regex fields)))
+      (mew-header-goto-end)
+      (save-restriction
+	(narrow-to-region (point-min) (point))
+	(goto-char (point-min))
+	(while (re-search-forward regex nil t)
+	  (beginning-of-line)
+	  (insert prefix)
+	  (forward-line))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Header insertion
 ;;;
 
-;; no need to inherit property because local-map is now set onto
-;; overlay. must insert before markers (i.e. mew-marker-header-end).
-
-(fset 'mew-header-insert (symbol-function 'insert-before-markers))
-
-(defun mew-header-insert-here (field value &optional noret)
+(defun mew-draft-header-insert (field value)
+  "No encoding. Just insert."
   (if (and value (stringp field))
-      (progn
-	(mew-header-insert field)
-	(mew-header-insert " ")
-	(if (stringp value) (mew-header-insert value))
-	(if (not noret) (mew-header-insert "\n")))))
+      (insert field " " value "\n")))
 
-(defun mew-header-insert-param-here (params)
-  (if (null params)
-      ()
-    (mew-header-insert (format "; %s" (car params)))
-    (setq params (cdr params))
-    (while params
-      (mew-header-insert (format ";\n\t%s" (car params)))
-      (setq params (cdr params)))))
-
-(defmacro mew-header-fill-here (field value)
+(defmacro mew-draft-header-fill (field value)
   (` (if (not (mew-header-existp (, field)))
-	 (mew-header-insert-here (, field) (, value)))))
+	 (mew-draft-header-insert (, field) (, value)))))
+
+(defun mew-header-insert (key value &optional params)
+  (if (and value (stringp key))
+      (let ((beg (point)) med par parname parval)
+	(insert key)
+  	(insert " ")
+	(setq med (point))
+	(if (string-match "^[\t -~]*$" value)
+	    (insert value)
+	  (mew-header-encode-text value nil (length key)))
+	(while params
+	  (setq par (car params))
+	  (setq parname (nth 0 par))
+	  (setq parval (nth 1 par))
+	  (setq params (cdr params))
+	  (insert ";")
+	  (cond
+	   ((string-match "^[-a-zA-Z0-9]+$" parval)
+	    ) ;; do nothing
+	   ((and (equal (mew-charset-guess-string parval) mew-us-ascii)
+		 (not (string-match "\"" parval)))
+	    (setq parval (concat "\"" parval "\"")))
+	   (t
+	    (setq parval (mew-param-encode parval))
+	    (setq parname (concat parname "*"))))
+	  (insert " " parname "=" parval))
+	(insert "\n")
+	(mew-header-fold-region beg (point) med))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -137,9 +163,7 @@
 ;; But 'domain-literal' (e.g. [1.2.3.4]) is meaningless.
 ;; So, the delimiters consists of SEP, SP, TAB, CRLF, "(", ")", and <">.
 ;; 
-;; 'qtext' retains. (It will be removed by mew-syntax-get-member.)
-;; 'qtext' allows linear-white-space but assumed that linear-white-space
-;; appears only after ",".
+;; 'qtext' remains. (It will be removed by mew-param-decode.)
 ;; ## how about 'quoted-pair'?
 ;;
 ;; 'comment' must be symmetric. That is, it must have the exact number of
@@ -153,14 +177,15 @@
 ;; meaningless.
 
 (defmacro mew-addrstr-parse-syntax-list-check-depth ()
-  (` (progn
-       (setq sep-cnt (1+ sep-cnt))
-       (if (>= sep-cnt mew-header-max-depth)
-	   (progn
-	     (message "Too many values. Truncate values over mew-header-max-depth.")
-	     (ding)
-	     (sit-for 2)
-	     (throw 'max nil))))))
+  '(progn
+     (setq sep-cnt (1+ sep-cnt))
+     (if (and (integerp mew-header-max-depth)
+	      (>= sep-cnt mew-header-max-depth))
+	 (progn
+	   (message "Too many values. Truncate values over mew-header-max-depth.")
+	   (ding)
+	   (sit-for 2)
+	   (throw 'max nil)))))
 
 (defun mew-addrstr-parse-syntax-list (str sep addrp)
   (let* ((i 0) (len (length str))
@@ -178,12 +203,30 @@
 	  (catch 'quote
 	    (while (< i len)
 	      (setq c (aref str i))
-	      (if (char-equal c ?\")
-		  (throw 'quote (progn
-				  (aset tmp tmp-cnt c)
-				  (setq tmp-cnt (1+ tmp-cnt))))
+	      (cond
+	       ((char-equal c ?\")
 		(aset tmp tmp-cnt c)
-		(setq tmp-cnt (1+ tmp-cnt)))
+		(setq tmp-cnt (1+ tmp-cnt))
+		(throw 'quote nil))
+	       ((char-equal c ?\n)
+		(setq i (1+ i))
+		(catch 'fold-quote
+		  (while (< i len)
+		    (setq c (aref str i))
+		    (cond
+		     ((or (char-equal c ?\t) (char-equal c 32))
+		      (setq i (1+ i)))
+		     ((char-equal c ?\")
+		      (aset tmp tmp-cnt c)
+		      (setq tmp-cnt (1+ tmp-cnt))
+		      (throw 'quote nil))
+		     (t
+		      (aset tmp tmp-cnt c)
+		      (setq tmp-cnt (1+ tmp-cnt))
+		      (throw 'fold-quote nil))))))
+	       (t
+		(aset tmp tmp-cnt c)
+		(setq tmp-cnt (1+ tmp-cnt))))
 	      (setq i (1+ i)))))
 	 ((char-equal c ?\()
 	  (setq par-cnt 1)
@@ -198,21 +241,27 @@
 		(setq par-cnt (1- par-cnt))
 		(if (equal par-cnt 0) (throw 'comment nil))))
 	      (setq i (1+ i)))))
-	 ((and addrp (char-equal c ?<))
-	  (let (rbeg rend)
-	    (setq i (1+ i))
-	    (setq rbeg i)
-	    (while (and (< i len) (not (char-equal (aref str i) ?>)))
+	 ((char-equal c ?<)
+	  (cond
+	   (addrp
+	    (let (rbeg rend)
+	      (setq i (1+ i))
+	      (setq rbeg i)
+	      (while (and (< i len) (not (char-equal (aref str i) ?>)))
+		(setq i (1+ i)))
+	      (setq rend i);; note: to be used for substring, so not 1-.
+	      ;; should not be nested but easy to implement...
+	      (setq ret (cons (car (mew-addrstr-parse-syntax-list
+				    (substring str rbeg rend) sep t))
+			      ret)))
+	    (while (and (< i len) (not (char-equal (aref str i) sep)))
 	      (setq i (1+ i)))
-	    (setq rend i);; note: to be used for substring, so not 1-.
-	    ;; should not be nested but easy to implement...
-	    (setq ret (cons (car (mew-addrstr-parse-syntax-list
-				  (substring str rbeg rend) sep t))
-			    ret)))
-	  (while (and (< i len) (not (char-equal (aref str i) sep)))
-	    (setq i (1+ i)))
-	  (setq tmp-cnt 0)
-	  (mew-addrstr-parse-syntax-list-check-depth))
+	    (setq tmp-cnt 0)
+	    (mew-addrstr-parse-syntax-list-check-depth))
+	   (t
+	    ;; just ignore
+	    (while (and (< i len) (not (char-equal (aref str i) ?>)))
+	      (setq i (1+ i))))))
 	 ((char-equal c ?\n))
 	 ((char-equal c ?\t))
 	 ((char-equal c 32))
@@ -263,6 +312,19 @@
       (mapconcat (function mew-header-get-value) (, field-list) ",")
       ?, t)))
 
+(defun mew-header-parse-address-list2 (field-list)
+  "Collect addresses from FIELD-LIST. 
+Remove anonymous addresses."
+  (let ((vals (mew-addrstr-parse-syntax-list
+	       (mapconcat (function mew-header-get-value) field-list ",")
+	       ?, t))
+	ret)
+    (while vals
+      (if (not (string-match mew-anonymous-recipients (car vals)))
+	  (setq ret (cons (car vals) ret)))
+      (setq vals (cdr vals)))
+    (nreverse ret)))
+
 ;;
 
 (defmacro mew-addrstr-extract-user-list (addr-list)
@@ -270,40 +332,50 @@
 
 (defun mew-addrstr-extract-user (addr)
   "Extracts username from ADDR"
-  (if (string-match "@.*:" addr)
-      (setq addr (substring addr (match-end 0) (length addr)))
+  (if (string-match "@.*:" addr) ;; xxx what's this?
+      (setq addr (substring addr (match-end 0) nil))
+    (setq addr (mew-replace-character addr 32 ?_))
     (setq addr (substring addr 0 (string-match "%" addr)))
     (setq addr (substring addr 0 (string-match "@" addr)))
-    (substring addr 0 (string-match ":;" addr))))
+    ;; just for refile:  "To: recipients:;" -> recipients
+    (setq addr (substring addr 0 (string-match mew-anonymous-recipients addr)))
+    ;; removing Notes domain
+    (setq addr (substring addr 0 (string-match "/" addr)))))
 
 ;;
 
+(defun mew-addrstr-append-domain (addr)
+  (if mew-addrbook-append-domain-p
+      (if (string-match "@" addr)
+	  addr
+	(concat addr "@" mew-mail-domain))
+    addr))
+
 (defun mew-addrstr-expand-alias (alias)
-  "Expand ALIS with mew-alias-alist"
-  (let ((assoc (mew-assoc-case-equal alias mew-alias-alist 0)))
-    (if assoc (cdr assoc) alias)))
+  (if (and mew-addrbook-unexpand-regex
+	   (string-match mew-addrbook-unexpand-regex alias))
+      (list alias) ;; not expand
+    (let ((addrs (mew-alias-get alias)))
+      (setq addrs (mapcar (function mew-chop) (mew-split addrs ?,)))
+      (setq addrs (delete "" addrs))
+      (mapcar (function mew-addrstr-append-domain) addrs))))
 
-(defun mew-addrstr-canonicalize-address (addr)
-  "Exapnd and canonicalize ADDR with mew-mail-domain."
-  (let ((eaddr (mew-addrstr-expand-alias addr)))
-    (if (or (null mew-mail-domain) (string= mew-mail-domain ""))
-	eaddr
-      (if (string-match "@" eaddr)
-	  eaddr
-	(concat eaddr "@" mew-mail-domain)))))
-
-(defmacro mew-addrstr-canonicalize-address-list (addr-list)
-  (` (mapcar (function mew-addrstr-canonicalize-address) (, addr-list))))
-
-(defmacro mew-header-canonicalize-address-list (field-list)
-  (` (mew-addrstr-canonicalize-address-list
-      (mew-header-parse-address-list (, field-list)))))
+(defun mew-addrstr-canonicalize-address (addr-str)
+  "Expand aliases for each addresses in ADDR-STR. 
+Then prepend mew-mail-domain if a domain part doesn't exist."
+  (let ((addrs (mapcar (function mew-chop) (mew-split addr-str ?,)))
+	addr ret)
+    (while addrs
+      (setq addr (car addrs))
+      (setq addrs (cdr addrs))
+      (setq ret (nconc ret (mew-addrstr-expand-alias addr))))
+    ret))
 
 (provide 'mew-header)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996, 1997, 1998 Mew developing team.
+;; Copyright (C) 1996, 1997, 1998, 1999 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
