@@ -2,11 +2,11 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Mar  2, 1997
-;; Revised: Aug 24, 1998
+;; Revised: Aug 30, 1999
 
 ;;; Code:
 
-(defconst mew-mark-version "mew-mark.el version 0.11")
+(defconst mew-mark-version "mew-mark.el version 0.12")
 
 (require 'mew)
 
@@ -20,18 +20,14 @@
    (cons mew-mark-multi  'mew-mark-keep-line)
    (cons mew-mark-review 'mew-mark-keep-line)
    (cons mew-mark-delete 'mew-mark-delete-line)
-   (cons mew-mark-refile 'mew-mark-delete-line)
-   )
-  )
+   (cons mew-mark-refile 'mew-mark-delete-line)))
 
 (defvar mew-mark-undo-switch
   (list 
    (cons mew-mark-multi  'mew-mark-unmark)
    (cons mew-mark-review 'mew-mark-unmark)
    (cons mew-mark-delete 'mew-mark-unmark)
-   (cons mew-mark-refile 'mew-mark-unrefile)
-   )
-  )
+   (cons mew-mark-refile 'mew-mark-unrefile)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -42,12 +38,10 @@
   (beginning-of-line)
   (let ((start (point)))
     (forward-line)
-    (delete-region start (point))
-    ))
+    (delete-region start (point))))
 
 (defun mew-mark-keep-line ()
-  () ;; do nothing
-  )
+  ()) ;; do nothing
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -57,18 +51,18 @@
 (defun mew-mark-unmark ()
   "Delete the mark on this message anyway"
   (save-excursion
-    (let ((buffer-read-only nil))
-      (beginning-of-line)
-      (re-search-forward mew-summary-message-regex)
-      (delete-char 1)
-      (insert " ")
-      (mew-highlight-unmark-line))))
+    (mew-elet
+     (beginning-of-line)
+     (if (re-search-forward mew-summary-message-regex nil t)
+	 (progn
+	   (delete-char 1)
+	   (insert " ")
+	   (mew-highlight-unmark-line))))))
 
 (defun mew-mark-unrefile ()
   "Delete refile state and delete the mark"
   (mew-refile-reset (mew-summary-message-number))
-  (mew-mark-unmark)
-  )
+  (mew-mark-unmark))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -81,9 +75,7 @@
     (beginning-of-line)
     (cond 
      ((looking-at (concat mew-summary-message-regex " \\|^$")) nil)
-     (t t)
-     )
-    ))
+     (t t))))
 
 (defun mew-summary-get-mark ()
   (let (mark)
@@ -94,23 +86,21 @@
 	(setq mark (mew-match 2)) ;; regex includes \( \)
 	(cond 
 	 ((string= " " mark) nil)
-	 (t (string-to-char mark))
-	 )
-	)
-      )))
+	 (t (string-to-char mark)))))))
 
 (defun mew-summary-mark-as (mark &optional force)
   "Mark this message if possible"
-  (let ((buffer-read-only nil))
-    (cond 
-     ((or force (null (mew-summary-marked-p)))
-      (beginning-of-line)
-      (re-search-forward mew-summary-message-regex)
-      (delete-char 1)
-      (insert (char-to-string mark))
-      (mew-highlight-mark-line mark))
-     (t (message "Already marked")))
-    ))
+  (mew-elet
+   (cond 
+    ((or force (null (mew-summary-marked-p)))
+     (save-excursion
+       (beginning-of-line)
+       (if (re-search-forward mew-summary-message-regex nil t)
+	   (progn
+	     (delete-char 1)
+	     (insert (char-to-string mark))
+	     (mew-highlight-mark-line mark)))))
+    (t (message "Already marked")))))
 
 (defmacro mew-mark-regex (mark)
   (` (concat mew-summary-message-regex
@@ -194,21 +184,21 @@ cons pairs of folder and number"
 
 ;; "@" in Summary mode
 (defun mew-summary-multi ()
-  "Put the multi the '@' mark on this message for 'F', 'M-s', 
-and 'M-t'. It can overlay the '*' mark. The cursor stays always."
+  "\\<mew-summary-mode-map>
+Put the multi the '@' mark on this message for '\\[mew-summary-multi-forward]', '\\[mew-summary-unshar]', 
+'\\[mew-summary-uudecode]', '\\[mew-summary-burst-multi]'. It can overlay the '*' mark. 
+The cursor stays always."
   (interactive)
-  (mew-summary-mark-weak mew-mark-multi)
-  )
+  (mew-summary-mark-weak mew-mark-multi))
 
 ;; "*" in Summary mode
 (defun mew-summary-review ()
-  "Put the review the '*' mark on this message. 
-Use 'N' or 'P' to jump to a message marked with '*'.
+  "\\<mew-summary-mode-map>Put the review the '*' mark on this message. 
+Use '\\[mew-summary-display-review-down]' or '\\[mew-summary-display-review-up]' to jump to a message marked with '*'.
 It can overlay '@'. The cursor stays always.
-See also 'mo', 'md', 'mr', and 'ma'."
+See also '\\[mew-summary-mark-refile]', '\\[mew-summary-mark-delete]', '\\[mew-summary-mark-regexp]', and '\\[mew-summary-mark-all]'."
   (interactive)
-  (mew-summary-mark-weak mew-mark-review)
-  )
+  (mew-summary-mark-weak mew-mark-review))
 
 (defun mew-summary-down-mark (mark)
   (forward-line)
@@ -225,8 +215,7 @@ See also 'mo', 'md', 'mr', and 'ma'."
   "Jump to the message marked with '*' below."
   (interactive)
   (if (mew-summary-down-mark mew-mark-review)
-      (if mew-summary-buffer-disp-msg ;; just seed up!
-	  (mew-summary-display))))
+      (mew-summary-display nil)))
 
 (defun mew-summary-up-mark (mark)
   (cond 
@@ -234,15 +223,13 @@ See also 'mo', 'md', 'mr', and 'ma'."
     t)
    (t 
     (message "No more marked message")
-    nil)
-   ))
+    nil)))
 
 (defun mew-summary-display-review-up ()
   "Jump to the message marked with '*' above."
   (interactive)
   (if (mew-summary-up-mark mew-mark-review)
-      (if mew-summary-buffer-disp-msg ;; just seed up!
-	  (mew-summary-display))))
+      (mew-summary-display nil)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -264,7 +251,7 @@ on the message. If it marks newly, displays the next message."
 	      ;; for C-x C-x
 	      (beginning-of-line)
 	      (push-mark (point) t t)
-	      (mew-summary-display-next)))
+	      (mew-summary-display-after mew-summary-mark-direction)))
       (while (and (> count 0) (not (eobp)))
 	(setq count (1- count))
 	(mew-summary-delete-one 'force)
@@ -288,8 +275,7 @@ on the message. If it marks newly, displays the next message."
       (if (or force (y-or-n-p "Already marked as refile. Delete it? "))
 	  (progn
 	    (mew-summary-undo-one force)
-	    (save-excursion (mew-summary-mark-as mew-mark-delete))
-	    ))
+	    (save-excursion (mew-summary-mark-as mew-mark-delete))))
       nil)
      ((or (equal mark mew-mark-review) (equal mark mew-mark-multi))
       (mew-summary-undo-one force)
@@ -297,8 +283,7 @@ on the message. If it marks newly, displays the next message."
       nil)
      (t ;; no mark
       (save-excursion (mew-summary-mark-as mew-mark-delete))
-      t)
-     )))
+      t))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -327,14 +312,14 @@ on the message. If it marks newly, displays the next message."
    (set-buffer-modified-p nil)))
 
 (defun mew-summary-undo-one (&optional force)
+  "Undo one mark here."
   (interactive)
   (cond 
    ((mew-summary-marked-p)
     (beginning-of-line)
     (looking-at (concat mew-summary-message-regex "\\([^ +0-9]\\)"))
     (funcall (cdr (assoc (string-to-char (mew-match 2))
-			 mew-mark-undo-switch)))
-    )
+			 mew-mark-undo-switch))))
    ((eobp) (or force (message "No message")))
    (t (or force (message "No mark")))))
 
@@ -347,8 +332,7 @@ on the message. If it marks newly, displays the next message."
 	  (message "Input mark : ")
 	  (setq cursor-in-echo-area t)
 	  (setq char (read-char))
-	  (message "Input mark : %s" (char-to-string char))
-	  )
+	  (message "Input mark : %s" (char-to-string char)))
       (setq cursor-in-echo-area ociea))
     (if (assoc char mew-mark-undo-switch)
 	(mew-summary-batch-unmark (list char) 'msg)
@@ -358,8 +342,7 @@ on the message. If it marks newly, displays the next message."
 (defun mew-summary-mark-undo-all ()
   "Unmark all message marked with 'o' or 'D'."
   (interactive)
-  (mew-summary-batch-unmark (list mew-mark-delete mew-mark-refile) 'msg)
-  )
+  (mew-summary-batch-unmark (list mew-mark-delete mew-mark-refile) 'msg))
 
 (defun mew-summary-batch-unmark (mark-list msg)
   (mew-decode-syntax-delete)
@@ -385,6 +368,8 @@ on the message. If it marks newly, displays the next message."
     (mew-summary-mark-region (point-min) (point-max))))
 
 (defun mew-summary-mark-region (beg end)
+  "Put the '*' mark onto all messages which are not marked between
+BEG and END."
   (interactive "r")
   (let ((regex (concat mew-summary-message-regex " ")))
     (save-excursion
@@ -397,11 +382,13 @@ on the message. If it marks newly, displays the next message."
 (defun mew-summary-mark-regexp (regex)
   "Put the '*' mark onto Mall messages matched to a regular expression."
   (interactive "sRegexp: ")
-  (save-excursion
-    (goto-char (point-min))
-    (while (re-search-forward regex nil t)
-      (if (not (mew-summary-marked-p))
-	  (mew-summary-mark-as mew-mark-review)))))
+  (if (not (equal regex ""))
+      (save-excursion
+        (goto-char (point-min))
+        (while (re-search-forward regex nil t)
+          (if (or (mew-summary-marked-p) (mew-in-decode-syntax-p))
+	      ()
+	    (mew-summary-mark-as mew-mark-review))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -415,11 +402,11 @@ on the message. If it marks newly, displays the next message."
       (if (not (re-search-forward regex nil t))
 	  (message "No marked messages")
 	(beginning-of-line)
-	(let ((buffer-read-only nil))
-	  (while (re-search-forward regex nil t)
-	    (delete-backward-char 1)
-	    (insert (char-to-string new-mark))
-	    (mew-highlight-mark-line new-mark)))))))
+	(mew-elet
+	 (while (re-search-forward regex nil t)
+	   (delete-backward-char 1)
+	   (insert (char-to-string new-mark))
+	   (mew-highlight-mark-line new-mark)))))))
    
 (defun mew-summary-mark-delete ()	;; * -> D
   "Put the delete mark onto all messages marked with '*'."
@@ -444,29 +431,64 @@ on the message. If it marks newly, displays the next message."
   (mew-summary-exchange-mark mew-mark-review mew-mark-multi)
   (mew-summary-exchange-mark mew-mark-tmp mew-mark-review))
 
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Cleaning trash
+;;;
+
+(defun mew-summary-clean-trash ()
+  "Remove all messages in +trash."
+  (interactive)
+  (if (not (yes-or-no-p (format "Remove all messages in %s? " mew-trash-folder)))
+      ()
+    (message "Removing all messages in %s ..." mew-trash-folder)
+    (let ((msgs (directory-files (mew-expand-folder mew-trash-folder)
+				 nil "^[0-9]+$"))
+	  (cfile (mew-expand-folder
+		  mew-trash-folder mew-summary-cache-file))
+	  (cbuf))
+      (mew-summary-imclean mew-trash-folder msgs)
+      (if (get-buffer mew-trash-folder)
+	  (save-excursion
+	    (set-buffer mew-trash-folder)
+	    (mew-erase-buffer)))
+      ;; remove and touch the cache file.
+      (if (file-exists-p cfile)
+	  (save-excursion
+	    (setq cbuf (find-file-noselect cfile))
+	    (set-buffer cbuf)
+	    (mew-erase-buffer)
+	    (write-region (point-min) (point-max) cfile nil 'no-msg)
+	    (set-buffer-modified-p nil)
+	    (kill-buffer cbuf))))
+    (message "Removing all messages in %s ... done" mew-trash-folder)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Processing marks
 ;;;
 
 (defun mew-mark-delete-region (begin end)
-  (let ((buffer-read-only nil)
-	(m (make-marker))
-	(regex (concat mew-summary-message-regex "\\([^ +0-9]\\)")))
-    (set-marker m end)
-    (goto-char begin)
-    (while (re-search-forward regex m t)
-      (funcall (cdr (assoc (string-to-char (mew-match 2))
-			   mew-mark-delete-switch))))
-    (set-buffer-modified-p nil)))
+  (mew-elet
+   (let ((m (make-marker))
+	 (regex (concat mew-summary-message-regex "\\([^ +0-9]\\)")))
+     (set-marker m end)
+     (goto-char begin)
+     (while (re-search-forward regex m t)
+       (funcall (cdr (assoc (string-to-char (mew-match 2))
+			    mew-mark-delete-switch))))
+     (set-buffer-modified-p nil))))
 
 (defun mew-summary-exec ()
-  "Process marked messages. To cancel the '*' mark, use 'u' or 'U'."
+  "\\<mew-summary-mode-map>
+Process marked messages. To cancel the '*' mark, use '\\[mew-summary-undo]' or '\\[mew-summary-undo-all]'."
   (interactive)
   (mew-summary-only
    (mew-summary-exec-region (point-min) (point-max))))
 
 (defun mew-summary-exec-current ()
+  "Process the current marked messages."
   (interactive)
   (mew-summary-only
    (mew-summary-goto-message)
@@ -482,6 +504,7 @@ on the message. If it marks newly, displays the next message."
 (defvar mew-summary-exec-error-msg nil)
 
 (defun mew-summary-exec-region (beg end)
+  "Process marked messages between BEG and END."
   (interactive "r")
   (mew-summary-only
    (if (not (mew-summary-exclusive-p))
@@ -504,8 +527,8 @@ on the message. If it marks newly, displays the next message."
 		 (setq mew-summary-buffer-process t)
 		 (message "Refiling and deleting ...")
 		 (mew-window-configure (current-buffer) 'summary)
-		 (mew-decode-syntax-delete)
 		 (mew-current-set 'message nil)
+		 (mew-decode-syntax-delete)
 		 ;; refiling and deleting...
 		 (if (setq err-folders (mew-summary-exec-sanity-check msgs))
 		     (progn
@@ -518,13 +541,13 @@ on the message. If it marks newly, displays the next message."
 		   (mew-summary-exec-delete src dels)
 		   ;; ending...
 		   (mew-mark-delete-region (point-min) (point-max))
-		   (if mew-summary-cache-use (mew-summary-folder-cache-save))
+		   (mew-summary-folder-cache-save src)
 		   (mew-refile-guess-save)
 		   (run-hooks 'mew-summary-exec-hook)
 		   (set-buffer-modified-p nil)
-		   (mew-summary-mode-line (current-buffer))
-		   (message "Refiling and deleting ... done")
+		   (mew-summary-reset-mode-line (current-buffer))
 		   (setq mew-summary-buffer-process nil)
+		   (message "Refiling and deleting ... done")
 		   ))) ;; end of let
 	   (quit
 	    (set-buffer-modified-p nil)
@@ -575,8 +598,7 @@ on the message. If it marks newly, displays the next message."
     ;; refile at once
     (while dsts-msgses
       (mew-summary-immv src (car dsts-msgses))
-      (setq dsts-msgses (cdr dsts-msgses)))
-    ))
+      (setq dsts-msgses (cdr dsts-msgses)))))
 
 (defun mew-summary-exec-delete (src dels)
   (if dels
@@ -589,8 +611,8 @@ on the message. If it marks newly, displays the next message."
 	      (setq rm-it t)))
 	 ((equal mew-msg-rm-policy 'uselist)
 	  (if (mew-member-match src mew-msg-rm-folder-list)
-	      (setq rm-it t)))
-	 )
+	      (setq rm-it t))))
+	;; (t ;; 'totrash)
 	(if rm-it
 	    (mew-summary-imclean src dels)
 	  (if (equal src mew-trash-folder)
@@ -604,9 +626,15 @@ on the message. If it marks newly, displays the next message."
   (let* ((dsts (car dsts-msgs)) ;; (+foo +bar)
 	 (msgs (cdr dsts-msgs)) ;; (1 2 3)
 	 (myselfp (mew-folder-member src dsts))
+	 (dsts-tmp dsts)
 	 msgs- msg srcfile dstfile dst num)
-    (if mew-use-imap
-	(mew-summary-start-im mew-prog-immv src dsts msgs)
+    (if (or mew-use-immv (mew-folder-remotep src)
+	    (catch 'remote
+	      (while dsts-tmp
+		(if (mew-folder-remotep (car dsts-tmp))
+		    (throw 'remote t))
+		(setq dsts-tmp (cdr dsts-tmp)))))
+	(mew-summary-im-start mew-prog-immv src dsts msgs t)
       (if myselfp
 	  ;; msg stays in the src folder with the same number
 	  (progn
@@ -645,8 +673,7 @@ on the message. If it marks newly, displays the next message."
 	  (setq dstfile (mew-expand-folder dst (int-to-string num)))
 	  (setq num (1+ num))
 	  (mew-link srcfile dstfile))
-	(mew-touch-folder dst))
-      )
+	(mew-touch-folder dst)))
     (if (not myselfp)
 	()
       (goto-char (point-min))
@@ -661,8 +688,8 @@ on the message. If it marks newly, displays the next message."
 	(setq msgs (cdr msgs))))))
 
 (defun mew-summary-imrm (src dels)
-  (if mew-use-imap
-      (mew-summary-start-im mew-prog-imrm src nil dels)
+  (if (or mew-use-immv (mew-folder-remotep src))
+      (mew-summary-im-start mew-prog-imrm src nil dels t)
     (let* (num srcfile dstfile)
       (setq num (string-to-int (mew-folder-new-message mew-trash-folder t)))
       ;; must be here after ensuring that +trash exists.
@@ -678,8 +705,8 @@ on the message. If it marks newly, displays the next message."
       (mew-touch-folder mew-trash-folder))))
 
 (defun mew-summary-imclean (src dels)
-  (if mew-use-imap
-      (mew-summary-start-im mew-prog-imclean src nil dels "--quiet=yes")
+  (if (or mew-use-immv (mew-folder-remotep src))
+      (mew-summary-im-start mew-prog-imclean src nil dels t "--quiet=yes")
     (let (file)
       (while dels
 	(setq file (mew-expand-folder src (car dels)))
@@ -688,50 +715,6 @@ on the message. If it marks newly, displays the next message."
 	    ;; if not, the marked line will be deleted anyway.
 	    (delete-file file))))))
 
-(defun mew-summary-start-im (imprg src dsts msgs &optional arg)
-  (let ((imarg (if arg (cons arg mew-prog-im-arg) mew-prog-im-arg)))
-    (or (listp dsts) (setq dsts (list dsts)))
-    (or (listp msgs) (setq msgs (list msgs)))
-    (setq mew-summary-imap-string nil)
-    (setq mew-summary-imap-process (apply (function start-process)
-					    imprg nil imprg
-					    (format "--src=%s" src)
-					    (append imarg dsts msgs)))
-    (set-process-filter 
-     mew-summary-imap-process 'mew-summary-imap-filter)
-    (set-process-sentinel
-     mew-summary-imap-process 'mew-summary-imap-sentinel)
-    (while mew-summary-imap-process
-      (if mew-xemacs-p
-	  (accept-process-output)
-	(sit-for 1)
-	;; accept-process-output or sleep-for is not enough
-	(discard-input)))))
-
-(defun mew-summary-imap-filter (process string)
-  (save-excursion
-    (setq mew-summary-imap-string
-	  (concat mew-summary-imap-string string))
-    (if (string-match "^Password for \\(.+\\):"
-		      mew-summary-imap-string)
-	(let ((userhost (mew-match 1 mew-summary-imap-string)))
-	  (setq mew-summary-imap-string "")
-	  (process-send-string
-	   process
-	   (format "%s\n"
-		   (mew-input-passwd
-		    (format "Enter password (%s) : " userhost))))))))
-
-(defun mew-summary-imap-sentinel (process event)
-  (let ((prog (process-name process)))
-    (and mew-summary-imap-string
-	 (string-match (format "^%s: \\([^\n]*\\)" prog)
-		       mew-summary-imap-string)
-	 (message "Refile error: %s"
-		  (mew-match 1 mew-summary-imap-string)))
-    (setq mew-summary-imap-process nil)
-    (setq mew-summary-imap-string nil)))
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Clean up marks!
@@ -771,7 +754,7 @@ Typically called by kill-emacs-hook."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997, 1998 Mew developing team.
+;; Copyright (C) 1997, 1998, 1999 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
