@@ -2,11 +2,11 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Mar 20, 1997
-;; Revised: Sep  3, 1998
+;; Revised: Aug 30, 1999
 
 ;;; Code:
 
-(defconst mew-mule0-version "mew-mule0.el version 0.09")
+(defconst mew-mule0-version "mew-mule0.el version 0.10")
 
 ;; In the context of Mew, 'charset' means MIME charset.
 ;; 'cs' means the internal representation of Emacs (was known as Mule).
@@ -15,20 +15,22 @@
 ;; User CS definitions
 ;;
 
-(defvar mew-cs-noconv       nil)
-(defvar mew-cs-noconv-eol   nil)
-(defvar mew-cs-autoconv     nil)
-(defvar mew-cs-7bit         nil)
-(defvar mew-cs-7bit-crlf    nil)
+(defvar mew-cs-dummy          nil)
+(defvar mew-cs-binary         nil)
+(defvar mew-cs-text-for-read  nil)
+(defvar mew-cs-text-for-write nil)
+(defvar mew-cs-autoconv       nil)
+(defvar mew-cs-7bit           nil)
+(defvar mew-cs-7bit-crlf      nil)
 
-(defvar mew-cs-mime-trans   nil)
-(defvar mew-cs-rfc822-trans nil)
-(defvar mew-cs-draft        nil)
-(defvar mew-cs-scan         nil)
-(defvar mew-cs-infile       nil)
-(defvar mew-cs-outfile      nil)
-(defvar mew-cs-virtual      nil)
-(defvar mew-cs-pick         nil)
+(defvar mew-cs-mime-trans     nil)
+(defvar mew-cs-rfc822-trans   nil)
+(defvar mew-cs-draft          nil)
+(defvar mew-cs-scan           nil)
+(defvar mew-cs-infile         nil)
+(defvar mew-cs-outfile        nil)
+(defvar mew-cs-virtual        nil)
+(defvar mew-cs-pick           nil)
 
 (defvar mew-cs-database
   '(("us-ascii"      (0)   nil   "7bit"             nil "Q")
@@ -123,7 +125,7 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997, 1998 Mew developing team.
+;; Copyright (C) 1997, 1998, 1999 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
