@@ -2,11 +2,16 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Mar 20, 1997
-;; Revised: Aug 24, 1998
+;; Revised: Aug 30, 1999
 
 ;;; Code:
 
-(defconst mew-mule2-version "mew-mule2.el version 0.08")
+(defconst mew-mule2-version "mew-mule2.el version 0.09")
+
+;; must be here
+(if (fboundp 'find-coding-system)
+    (fset 'mew-coding-system-p (symbol-function 'find-coding-system))
+  (fset 'mew-coding-system-p (symbol-function 'coding-system-p)))
 
 ;; In the context of Mew, 'charset' means MIME charset.
 ;; 'cs' means the internal representation of Emacs (was known as Mule).
@@ -15,19 +20,76 @@
 ;; User CS definitions
 ;;
 
-(defvar mew-cs-noconv       '*noconv*)
-(defvar mew-cs-noconv-eol   '*noconv*) ;; UNIX, for Win see mew.dot.emacs
-(defvar mew-cs-autoconv     '*autoconv*)
-(defvar mew-cs-7bit         '*iso-2022-ss2-7*)
-(defvar mew-cs-7bit-crlf    '*iso-2022-ss2-7*dos)
-(defvar mew-cs-mime-trans   '*iso-2022-ss2-7*)
-(defvar mew-cs-rfc822-trans '*iso-2022-ss2-7*) 
-(defvar mew-cs-draft        '*iso-2022-ss2-7*)
-(defvar mew-cs-scan         '*ctext*)
-(defvar mew-cs-infile       '*autoconv*)
-(defvar mew-cs-outfile      '*iso-2022-ss2-7*)
-(defvar mew-cs-virtual      '*iso-2022-ss2-7*unix) ;; ^M as it is
-(defvar mew-cs-pick         '*euc-japan*)
+(define-ccl-program mew-ccl-*-to-lf
+  '(1
+    ((read r0)
+     (loop
+      (if (r0 == 13)
+	  ((write 10)
+	   (read-if (r1 == 10)
+		    (read r0)
+		    (r0 = (r1 + 0)))
+	   (repeat))
+	(write-read-repeat r0)))))
+  "Read text with any eol converting to the LF eol.")
+
+(define-ccl-program mew-ccl-lf-to-lf
+  '(1
+    ((read r0)
+     (loop
+      (write-read-repeat r0))))
+  "Write text with the LF eol.")
+
+(define-ccl-program mew-ccl-lf-to-crlf
+  '(2
+    ((loop
+      (read-if (r0 == 10)
+	       (write 13))
+      (write r0)
+      (repeat))))
+  "Write text with the CRLF eol.")
+
+(define-ccl-program mew-ccl-lf-to-cr
+  '(1
+    ((loop
+      (read-if (r0 == 10)
+	       (write 13)
+	       (write r0))
+      (repeat))))
+  "Write text with the CR eol.")
+
+(make-coding-system 'mew-cs-text
+		    4 ?= "No conversion" nil
+		    (cons mew-ccl-*-to-lf mew-ccl-lf-to-lf))
+
+(make-coding-system 'mew-cs-text-lf
+		    4 ?= "No conversion" nil
+		    (cons mew-ccl-*-to-lf mew-ccl-lf-to-lf))
+
+(make-coding-system 'mew-cs-text-crlf
+		    4 ?= "No conversion" nil
+		    (cons mew-ccl-*-to-lf mew-ccl-lf-to-crlf))
+
+(make-coding-system 'mew-cs-text-cr
+		    4 ?= "No conversion" nil
+		    (cons mew-ccl-*-to-lf mew-ccl-lf-to-cr))
+
+(defvar mew-cs-dummy          '*noconv*)
+(defvar mew-cs-binary         '*noconv*)
+(defvar mew-cs-text-for-read  'mew-cs-text)
+(defvar mew-cs-text-for-write 'mew-cs-text-lf)
+(defvar mew-cs-autoconv       '*autoconv*)
+(defvar mew-cs-7bit           '*iso-2022-ss2-7*)
+(defvar mew-cs-7bit-crlf      '*iso-2022-ss2-7*dos)
+(defvar mew-cs-mime-trans     '*iso-2022-ss2-7*)
+(defvar mew-cs-rfc822-trans   '*iso-2022-ss2-7*) 
+(defvar mew-cs-draft          '*iso-2022-ss2-7*)
+(defvar mew-cs-scan           '*ctext*)
+(defvar mew-cs-infile         '*autoconv*)
+(defvar mew-cs-outfile        '*iso-2022-ss2-7*)
+(defvar mew-cs-virtual        (if (mew-coding-system-p '*ctext*unix)
+				  '*ctext*unix '*ctext*)) ;; ^M as it is
+(defvar mew-cs-pick           '*euc-japan*)
 
 (defvar mew-cs-database
   '(("us-ascii"    (0)             nil           "7bit"
@@ -54,18 +116,26 @@
                                    *tis620*      "B")
     ("iso-2022-jp" (0 138 146 144) *iso-2022-jp* "7bit"
                                    *iso-2022-jp* "B")
+    ("euc-kr"      (0 147)         *euc-kr*      "base64"
+                                   *euc-kr*      "B")
     ("iso-2022-kr" (0 147)         *iso-2022-kr* "7bit"
                                    *euc-kr*      "B")
-    ("cn-gb"       (0 145 160)     *euc-china*   "base64"
+    ("gb2312"      (0 145)         *euc-china*   "base64"
                                    *euc-china*   "B")
+    ("hz-gb-2312"  (0 145)         *hz*          "base64"
+                                   *hz*          "B")
+    ("big5"        (0 152 153)     *big5*        "base64"
+                                   *big5*        "B")
     ("iso-2022-jp-2" (0 138 146 144 145 147 148 129 134) 
      *iso-2022-ss2-7* "7bit" *iso-2022-ss2-7* "B")
     ;; charset-to-cs purpose only
-    ("euc-jp"         nil *euc-japan*)
-    ("shift_jis"      nil *sjis*)
-    ("cn-gb"          nil *euc-china*)
-    ("cn-big5"        nil *big5*)
-    ("iso-2022-int-1" nil *iso-2022-int-1*)
+    ("euc-jp"          nil *euc-japan*)
+    ("shift_jis"       nil *sjis*)
+    ("cn-gb"           nil *euc-china*) ;; the same as gb2312 above
+    ("cn-big5"         nil *big5*)      ;; the same as big5 above
+;   ("iso-2022-cn"     nil xxx)
+;   ("iso-2022-cn-ext" nil xxx)
+    ("iso-2022-int-1"  nil *iso-2022-int-1*)
     ))
 
 ;;
@@ -181,7 +251,7 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997, 1998 Mew developing team.
+;; Copyright (C) 1997, 1998, 1999 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
