@@ -2,11 +2,16 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Mar 20, 1997
-;; Revised: Aug 24, 1998
+;; Revised: Aug 30, 1999
 
 ;;; Code:
 
-(defconst mew-mule3-version "mew-mule3.el version 0.13")
+(defconst mew-mule3-version "mew-mule3.el version 0.14")
+
+;; must be here
+(if (fboundp 'find-coding-system)
+    (fset 'mew-coding-system-p (symbol-function 'find-coding-system))
+  (fset 'mew-coding-system-p (symbol-function 'coding-system-p)))
 
 ;; In the context of Mew, 'charset' means MIME charset.
 ;; 'cs' means the internal representation of Emacs (was known as Mule).
@@ -17,19 +22,34 @@
 
 ;; iso-2022-7bit-ss2 is iso-2022-jp-2
 
-(defvar mew-cs-noconv       'binary)
-(defvar mew-cs-noconv-eol   'no-conversion)
-(defvar mew-cs-autoconv     'undecided)
-(defvar mew-cs-7bit         'iso-2022-7bit-ss2)
-(defvar mew-cs-7bit-crlf    'iso-2022-7bit-ss2-dos)
-(defvar mew-cs-mime-trans   'iso-2022-7bit-ss2)
-(defvar mew-cs-rfc822-trans 'iso-2022-7bit-ss2)
-(defvar mew-cs-draft        'iso-2022-7bit-ss2)
-(defvar mew-cs-scan         'ctext)
-(defvar mew-cs-infile       'undecided) ;; guess in +draft
-(defvar mew-cs-outfile      'iso-2022-7bit-ss2)
-(defvar mew-cs-virtual      'iso-2022-7bit-ss2-unix) ;; ^M as it is
-(defvar mew-cs-pick         'euc-jp)
+(cond
+ (mew-xemacs-p
+  (copy-coding-system 'no-conversion      'mew-cs-text)
+  (copy-coding-system 'no-conversion-unix 'mew-cs-text-lf)
+  (copy-coding-system 'no-conversion-dos  'mew-cs-text-crlf)
+  (copy-coding-system 'no-conversion-mac  'mew-cs-text-cr))
+ (t
+  (define-coding-system-alias 'mew-cs-text      'raw-text)
+  (define-coding-system-alias 'mew-cs-text-lf   'raw-text-unix)
+  (define-coding-system-alias 'mew-cs-text-crlf 'raw-text-dos)
+  (define-coding-system-alias 'mew-cs-text-cr   'raw-text-mac)))
+
+(defvar mew-cs-dummy          'binary)
+(defvar mew-cs-binary         'binary)
+(defvar mew-cs-text-for-read  'mew-cs-text)
+(defvar mew-cs-text-for-write 'mew-cs-text-lf)
+(defvar mew-cs-autoconv       'undecided)
+(defvar mew-cs-7bit           'iso-2022-7bit-ss2)
+(defvar mew-cs-7bit-crlf      'iso-2022-7bit-ss2-dos)
+(defvar mew-cs-mime-trans     'iso-2022-7bit-ss2)
+(defvar mew-cs-rfc822-trans   'iso-2022-7bit-ss2)
+(defvar mew-cs-draft          'iso-2022-7bit-ss2)
+(defvar mew-cs-scan           'ctext)
+(defvar mew-cs-infile         'undecided) ;; guess in +draft
+(defvar mew-cs-outfile        'iso-2022-7bit-ss2)
+(defvar mew-cs-virtual        (if (mew-coding-system-p 'ctext-unix)
+				  'ctext-unix 'ctext)) ;; ^M as it is
+(defvar mew-cs-pick           'euc-jp)
 
 (defvar mew-cs-database
   '(("us-ascii"    (ascii)                    nil        "7bit"
@@ -52,27 +72,35 @@
                                               iso-8859-8 "B")
     ("iso-8859-9"  (ascii latin-iso8859-9)    iso-8859-9 "quoted-printable"
                                               iso-8859-9 "Q")
-    ("tis-620"     (ascii thai-tis620)        th-tis620  "base64"
-                                              th-tis620  "B")
+    ("tis-620"     (ascii thai-tis620)        tis620     "base64"
+                                              tis620     "B")
     ("iso-2022-jp"
      (ascii latin-jisx0201 japanese-jisx0208 japanese-jisx0208-1978)
                                               iso-2022-jp "7bit"
                                               iso-2022-jp "B")
+    ("euc-kr" (ascii korean-ksc5601)          euc-kr      "base64"
+                                              euc-kr      "B")
     ("iso-2022-kr" (ascii korean-ksc5601)     iso-2022-kr "7bit"
-                                              euc-korea   "B")
-    ("cn-gb" (ascii chinese-gb2312 chinese-sisheng)
-                                              cn-gb-2312 "base64"
-                                              cn-gb-2312 "B")
+                                              euc-kr      "B")
+    ("gb2312" (ascii chinese-gb2312)          cn-gb-2312  "base64"
+                                              cn-gb-2312  "B")
+    ("hz-gb-2312" (ascii chinese-gb2312)      hz-gb-2312  "7bit"
+                                              hz-gb-2312  "B")
+    ("big5" (ascii chinese-big5-1 chinese-big5-2)
+                                              chinese-big5 "base64"
+                                              chinese-big5 "B")
     ("iso-2022-jp-2" 
      (ascii
       latin-jisx0201 japanese-jisx0208 japanese-jisx0208-1978 japanese-jisx0212
       chinese-gb2312 korean-ksc5601 latin-iso8859-1 greek-iso8859-7)
-                                             iso-2022-7bit-ss2 "7bit"
-                                             iso-2022-7bit-ss2 "B")
+                                              iso-2022-7bit-ss2 "7bit"
+                                              iso-2022-7bit-ss2 "B")
     ;; charset-to-cs purpose only
     ("euc-jp"          nil euc-japan)
     ("shift_jis"       nil shift_jis)
-    ("cn-big5"         nil chinese-big5)
+    ("cn-gb"           nil cn-gb-2312)   ;; the same as gb2312 above
+    ("cn-big5"         nil chinese-big5) ;; the same as big5 above
+    ("iso-2022-cn"     nil iso-2022-cn)
     ("iso-2022-cn-ext" nil iso-2022-cn-ext)
     ("iso-2022-int-1"  nil iso-2022-int-1)
     ))
@@ -124,6 +152,7 @@
 
 (defmacro mew-piolet (read write &rest body)
   `(let ((call-process-hook nil)
+ 	 (file-name-coding-system ,write) ;; for imgrep
 	 (coding-system-for-read  ,read)
 	 (coding-system-for-write ,write))
      ,@body))
@@ -146,21 +175,7 @@
 ;; Post conversion
 ;;
 
-(defvar mew-postconv-to-convregion
-  '((viqr-post-read-conversion . viet-decode-viqr-region)
-    (thai-post-read-conversion . thai-compose-region)
-    (tibetan-post-read-conversion . tibetan-compose-region)
-    (in-is13194-devanagari-post-read-conversion .devanagari-compose-from-is13194-region))
-  "Alist of post-read-conversion and compose-region pair")
-
-
-(defun mew-cs-post-conv (cs)
-  (let (post-conv)
-    (if (fboundp (function coding-system))
-	(setq post-conv (coding-system-get cs 'post-read-conversion))
-      (setq post-conv (get cs 'post-read-conversion)))
-    (if post-conv
-	(cdr (assoc post-conv mew-postconv-to-convregion)))))
+(defun mew-cs-post-conv (cs) ()) ;; not necessary because internally applied
 
 ;;
 ;;
@@ -184,7 +199,7 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997, 1998 Mew developing team.
+;; Copyright (C) 1997, 1998, 1999 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
