@@ -2,11 +2,11 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Oct  2, 1996
-;; Revised: Aug 24, 1998
+;; Revised: Aug 30, 1999
 
 ;;; Code:
 
-(defconst mew-pick-version "mew-pick.el version 0.16")
+(defconst mew-pick-version "mew-pick.el version 0.17")
 
 (require 'mew)
 
@@ -37,7 +37,7 @@ then list them up."
 
 (defun mew-summary-search-mark (&optional arg)
   "Pick messages according to a pick pattern which you input, 
-then put the '*' mark onto them. If called with C-u, target is
+then put the '*' mark onto them. If called with \"\\[universal-argument]\", target is
 the messages in the region."
   (interactive "P")
   (mew-summary-only
@@ -70,11 +70,12 @@ the messages in the region."
 	(message "Marking messages ... ")
 	(goto-char r1)
 	(while (and range (< (point) r2))
-	  (if (re-search-forward (format "^[ ]*%s[^0-9]+" (car range)) nil t)
+	  (if (re-search-forward (format "^[ ]*%s[^0-9]" (car range)) nil t)
 	      (if (not (mew-summary-marked-p))
 		  (mew-summary-mark-as mew-mark-review)))
 	  (setq range (cdr range)))
 	(beginning-of-line)
+	(set-buffer-modified-p nil)
 	(message "Marking messages ... done")))))
 
 (defun mew-pick-define-macro (str1 str2)
@@ -198,19 +199,24 @@ the messages in the region."
 			   " "
 			   (substring string (match-end 0) nil)))
       (setq i (+ (match-beginning 0) 3)))
-    (mew-cs-encode-string string mew-cs-pick)))
+    string))
 
 (defun mew-summary-pick (folder pattern &optional range)
-  (let (msgs)
+  (let (arg msgs)
     (setq range (or range "all"))
     (save-excursion
       (mew-set-buffer-tmp)
       ;; input(result) from pick is line-based stream...
-      (mew-piolet
-       mew-cs-autoconv mew-cs-noconv
-       (mew-im-call-process nil mew-prog-imgrep
-			    (format "--src=%s" folder)
-			    (format "--expression=%s" pattern) range))
+      (cond
+       ((equal pattern mew-pick-duplicate-msgid)
+	(setq arg "--dupchecktarget=message-id"))
+       ((equal pattern mew-pick-duplicate-subj-msgid)
+	(setq arg "--dupchecktarget=message-id+subject"))
+       (t
+	(setq arg (format "--expression=%s" pattern))))
+      (mew-piolet mew-cs-autoconv mew-cs-pick
+	(mew-im-call-process nil mew-prog-imgrep
+			     (format "--src=%s" folder) arg range))
       (goto-char (point-min))
       (if (search-forward "imgrep: no message" nil t)
 	  (progn
@@ -231,7 +237,7 @@ the messages in the region."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996, 1997, 1998 Mew developing team.
+;; Copyright (C) 1996, 1997, 1998, 1999 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
