@@ -1,20 +1,21 @@
 ;;; mew-unix.el -- MIME content type for UNIX
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
-;; Created: Dec 4, 1997
-;; Revised: Aug 24, 1998
+;; Created: Dec  4, 1997
+;; Revised: Aug 30, 1999
 
 ;;; Code:
 
-(defconst mew-unix "mew-unix.el version 0.02")
+(defconst mew-unix "mew-unix.el version 0.03")
 
 ;;
 ;;
 ;;
 
-(defvar mew-prog-plain '(mew-mime-text/plain () nil))
-(defvar mew-prog-html '(mew-mime-text/html () nil))
-(defvar mew-prog-text '(mew-mime-text/plain () nil))
+(defvar mew-prog-plain    '(mew-mime-text/plain    () nil))
+(defvar mew-prog-html     '(mew-mime-text/html     () nil))
+(defvar mew-prog-enriched '(mew-mime-text/enriched () nil))
+(defvar mew-prog-text     '(mew-mime-text/plain    () nil))
 (defvar mew-prog-audio
   (list mew-prog-shell (list mew-prog-shell-arg "cat - > /dev/audio") nil))
 (defvar mew-prog-audio2
@@ -23,6 +24,10 @@
 			      (valid-image-instantiator-format-p 'gif))
 			 '(mew-mime-image/gif () nil)
 		       '("xv" ("-geometry" "+0+0") t)))
+(defvar mew-prog-tiff (if (and window-system mew-xemacs-p
+			       (valid-image-instantiator-format-p 'tiff));;xxx
+			  '(mew-mime-image/tiff () nil)
+			'("xv" ("-geometry" "+0+0") t)))
 (defvar mew-prog-jpeg (if (and window-system mew-xemacs-p
 			      (valid-image-instantiator-format-p 'jpeg))
 			  '(mew-mime-image/jpeg () nil)
@@ -46,7 +51,8 @@
 (defvar mew-prog-rfc822 '(mew-mime-message/rfc822 () nil))
 (defvar mew-prog-external-body '(mew-mime-external-body () nil))
 (defvar mew-prog-delivery-status '(mew-mime-text/plain () nil))
-(defvar mew-prog-postscript '("ghostview" ("-geom" "+0+0") t))
+(defvar mew-prog-postscript '("ghostview" ("-geometry" "+0+0") t))
+(defvar mew-prog-pdf '("acroread" ("-geometry" "+0+0") t))
 (defvar mew-prog-pgp-keys '(mew-mime-pgp-keys () nil))
 (defvar mew-prog-octet-stream '(mew-mime-application/octet-stream () nil))
 
@@ -54,7 +60,7 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996, 1997, 1998 Mew developing team.
+;; Copyright (C) 1996, 1997, 1998, 1999 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
