@@ -1,8 +1,8 @@
-;;; mew.el --- Messaging in the Emacs World
+;; mew.el --- Messaging in the Emacs World
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
-;; Created: Mar 22, 1994
-;; Revised: Sep  5, 1998
+;; Created: Mar 23, 1994
+;; Revised: Oct 15, 1999
 
 ;;; Commentary:
 
@@ -16,6 +16,17 @@
 ;;	(setq mew-mail-domain-list '("your mail domain"))
 ;;	(setq mew-icon-directory "icon directory")
 ;;
+;; Optional setup (e.g. for C-xm):
+;;	(autoload 'mew-user-agent-compose "mew" nil t)
+;;	(if (boundp 'mail-user-agent)
+;;	    (setq mail-user-agent 'mew-user-agent))
+;;	(if (fboundp 'define-mail-user-agent)
+;;	    (define-mail-user-agent
+;;	      'mew-user-agent
+;;	      'mew-user-agent-compose
+;;	      'mew-draft-send-letter
+;;	      'mew-draft-kill
+;;	      'mew-send-hook))
 
 ;;; Code:
 
@@ -24,7 +35,10 @@
 ;;; Mew version
 ;;;
 
-(defconst mew-version "Mew version 1.93")
+(defconst mew-version-number "1.94.1"
+  "Version number for this version of Mew.")
+(defconst mew-version (format "Mew version %s" mew-version-number)
+  "Version string for this version of Mew.")
 (provide 'mew)
 (require 'mew-vars)
 (require 'mew-func)
@@ -52,15 +66,15 @@
   "Execute Mew. If 'mew-auto-get' is 't', messages stored in your
 spool are fetched to the +inbox folder and messages in the +inbox
 folder are listed up in Summary mode. If 'mew-auto-get' is 'nil', list
-up messages in the inbox folder. If 'C-u' is specified, perform this
+up messages in the inbox folder. If '\\[universal-argument]' is specified, perform this
 function thinking that 'mew-auto-get' is reversed."
   (interactive "P")
+  (mew-window-push)
+  (if (null mew-mail-path) (mew-init))
   (let ((auto (if arg (not mew-auto-get) mew-auto-get)))
-    (mew-window-push)
-    (if (null mew-mail-path) (mew-init))
     (if auto
 	(mew-summary-get)
-      (mew-summary-goto-folder t mew-inbox-folder))))
+      (mew-summary-goto-folder t (mew-inbox-folder)))))
 
 (defun mew-send (&optional to cc subject)
   "Execute Mew then prepare a draft. This may be used as library
@@ -68,10 +82,43 @@ function."
   (interactive)
   (mew-current-set 'window (current-window-configuration))
   (if (null mew-mail-path) (mew-init))
-  ;; to ensure to back to *scratch*
-  (if (get-buffer mew-buffer-hello) (kill-buffer mew-buffer-hello))
   (mew-summary-send to cc subject))
 
+(defun mew-user-agent-compose (&optional to subject other-headers continue
+                                             switch-function yank-action
+                                             send-actions)
+  "Set up mail composition draft with Mew.
+This is 'mail-user-agent' entry point to Mew.
+
+The optional arguments TO and SUBJECT specify recipients and the
+initial Subject field, respectively.
+
+OTHER-HEADERS is an alist specifying additional
+header fields.  Elements look like (HEADER . VALUE) where both
+HEADER and VALUE are strings.
+
+A Draft buffer is prepared according to SWITCH-FUNCTION.
+
+CONTINUE, YANK-ACTION and SEND-ACTIONS are ignored."
+  (if (null mew-mail-path) (mew-init))
+  (let* ((draft (mew-folder-new-message mew-draft-folder))
+	 (attachdir (mew-attachdir draft)))
+    (mew-current-set 'window (current-window-configuration))
+    (mew-window-configure (current-buffer) 'draft)
+    (mew-summary-prepare-draft
+     (if switch-function
+	 (let ((special-display-buffer-names nil)
+	       (special-display-regexps nil)
+	       (same-window-buffer-names nil)
+	       (same-window-regexps nil))
+	   (funcall switch-function (find-file-noselect draft)))
+       (switch-to-buffer (find-file-noselect draft)))
+     (mew-draft-rename draft)
+     (mew-delete-directory-recursively attachdir)
+     (mew-draft-header subject nil to nil nil nil nil other-headers)
+     (mew-draft-mode)
+     (run-hooks 'mew-draft-mode-newdraft-hook))))
+
 ;;;
 ;;; Functions for boot time
 ;;;
@@ -79,25 +126,30 @@ function."
 (defun mew-init ()
   (mew-hello)
   (message "Setting Mew world ...")
+  (run-hooks 'mew-env-hook)
   (mew-set-environment)
-  (if mew-use-bbdb (require 'bbdb-com)) ;; bbdb is implicitly required
   (run-hooks 'mew-init-hook)
   (mew-status-update t)
+  (mew-passwd-setup)
+  (if (get-buffer mew-buffer-hello) (kill-buffer mew-buffer-hello))
   (message "Setting Mew world ... done"))
 
 (defun mew-set-environment (&optional no-dir)
   (let (error-message)
     (condition-case nil
 	(progn
-	  ;; version check
-	  (if (not (string-match "^18" emacs-version))
-	      ()
+	  ;; sanity check
+	  (cond
+	   ((string-match "^18" emacs-version)
 	    (setq error-message "Not support Emacs 18 nor Mule 1\n")
 	    (error ""))
+	   ((null mew-mail-domain-list)
+	    (setq error-message "Must set 'mew-mail-domain-list'")
+	    (error "")))
 	  ;; initialize IM variables
 	  (if (setq error-message (mew-config-init)) (error ""))
 	  ;; initializing
-	  (mew-buffers-init)
+	  (or no-dir (mew-buffers-init))
 	  (or no-dir (mew-temp-dir-init))
 	  (mew-mark-init)
 	  (mew-refile-init))
@@ -112,23 +164,26 @@ function."
        (error "Mew found some errors above.")))))
 
 (defun mew-status-update (arg)
-  "Update the list of aliases. If you type 'C-u Z' the list of folders
-are also updated in addition to that of aliases.  If 
-'mew-use-folders-file-p' is t, the list of folders is stored in
-\"~/Mail/.folders\"."
+  "Read Addrbook and update its information. If executed with '\\[universal-argument]',
+information of folders is also updated in addition to that of
+Addrbook. If 'mew-use-folders-file-p' is 't', the list of
+folders is stored in '~/Mail/.folders'. The default value is 't'."
   (interactive "P")
   (message "Updating status ... ")
   (if (interactive-p) (mew-set-environment 'no-dir))
   (if arg (mew-folder-setup nil (interactive-p)))
-  (mew-alias-setup)
-  (mew-petname-setup)
+  (mew-addrbook-setup)
   (mew-pgp-set-version)
   (mew-highlight-face-setup mew-highlight-header-face-list)
   (mew-highlight-face-setup mew-highlight-body-face-list)
   (mew-highlight-face-setup mew-highlight-mark-face-list)
   (mew-highlight-make-keywords-regex)
+  (mew-uniq-variables)
   (message "Updating status ...   done"))
 
+(defun mew-uniq-variables ()
+  (setq mew-mime-content-type-list (mew-uniq-list mew-mime-content-type-list)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Config
@@ -139,11 +194,13 @@ are also updated in addition to that of aliases.  If
 (defvar mew-queue-path nil)
 (defvar mew-inbox-folder nil)
 (defvar mew-draft-folder nil)
+(defvar mew-addrbook-file nil)
 (defvar mew-alias-file nil)
 (defvar mew-petname-file nil)
 (defvar mew-trash-folder nil)
 (defvar mew-imap-account nil)
 (defvar mew-config-cases nil)
+(defvar mew-config-case-inbox nil)
 
 (defvar mew-path-alist
   '(("MailPath" . mew-mail-path)
@@ -152,10 +209,12 @@ are also updated in addition to that of aliases.  If
     ("InboxFolder" . mew-inbox-folder)
     ("DraftFolder" . mew-draft-folder)
     ("TrashFolder" . mew-trash-folder)
-    ("AliasFile"   . mew-alias-file)
-    ("PetnameFile" . mew-petname-file)
+    ("AddrBookFile" . mew-addrbook-file)
+    ("AliasFile"    . mew-alias-file)
+    ("PetnameFile"  . mew-petname-file)
     ("ImapAccount" . mew-imap-account)
-    ("ConfigCases" . mew-config-cases)))
+    ("ConfigCases" . mew-config-cases)
+    ("ConfigCaseInbox" . mew-config-case-inbox)))
 
 (defvar mew-draft-mime-folder nil)
 
@@ -172,7 +231,7 @@ are also updated in addition to that of aliases.  If
     (goto-char (point-min))
     (let (key value pair)
       (while (not (eobp))
-	(if (looking-at "^\\(.+\\)=\\(.+\\)$")
+	(if (looking-at "^\\([^=\n]+\\)=\\(.+\\)$")
 	    ;; Petname may be null string.
 	    (progn
 	      (setq key (mew-match 1))
@@ -181,7 +240,15 @@ are also updated in addition to that of aliases.  If
 		  (set (cdr pair) value))))
 	(forward-line)))
     (if mew-config-cases 
-	(setq mew-config-list (mew-split mew-config-cases ?,)))
+	(setq mew-config-list (mew-split mew-config-cases ?,))
+      (setq mew-config-list (list mew-config-default)))
+    (if mew-config-case-inbox
+	(let (tmp)
+	  (setq mew-config-case-inbox (mew-split mew-config-case-inbox ?,))
+	  (while mew-config-case-inbox
+	    (setq tmp (cons (mew-split (car mew-config-case-inbox) ?:) tmp))
+	    (setq mew-config-case-inbox (cdr mew-config-case-inbox)))
+	  (setq mew-config-case-inbox (nreverse tmp))))
     (let (errmsg)
       (mew-config-error mew-mail-path "MailDir")
       (mew-config-error mew-news-path "NewsDir")
@@ -194,9 +261,9 @@ are also updated in addition to that of aliases.  If
 	(setq mew-draft-mime-folder
 	      (concat (file-name-as-directory mew-draft-folder) "mime"))
 	(mew-config-set-modes (list mew-mail-path mew-news-path))
-	(mew-config-create-folders (list mew-inbox-folder
-					 mew-draft-folder
-					 mew-trash-folder))
+	(mew-config-create-folders (cons mew-draft-folder
+					 (cons mew-trash-folder
+					       (mew-inbox-folders))))
 	nil)))) ;; return value
 
 (defun mew-config-set-modes (dirs)
@@ -205,7 +272,7 @@ are also updated in addition to that of aliases.  If
       (setq dir (file-chase-links (car dirs)))
       (setq dirs (cdr dirs))
       (if (file-exists-p dir)
-	  (if (/= mew-folder-mode (mew-get-file-modes dir))
+	  (if (/= mew-folder-mode (mew-file-get-mode dir))
 	      (set-file-modes dir mew-folder-mode))))))
 
 (defun mew-config-create-folders (folders)
@@ -227,74 +294,7 @@ are also updated in addition to that of aliases.  If
   (setq mew-petname-file nil)
   (setq mew-trash-folder nil)
   (setq mew-imap-account nil)
-  (setq mew-config-cases nil)
-  )
-
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
-;;; e-mail address alias
-;;;
-
-(defvar mew-alias-alist nil)
-
-(defun mew-alias-setup ()
-  (if mew-alias-file
-      (setq mew-alias-alist (mew-alias-make-alist))))
-
-(defun mew-alias-make-alist ()
-  "Make alias alist with association of (alias . expantion).
-Currently, only \"user: user@domain\" syntax is supported."
-  (save-excursion
-    (let* ((case-fold-search t)
-	   (pars (mew-split mew-alias-file ?,)) ;; parents
-	   (files pars) ;; included
-	   par chr path alias expn alist)
-      (mew-set-buffer-tmp)
-      ;; include parents files
-      (while pars
-	(setq par (car pars))
-	(setq pars (cdr pars))
-	(if (not (file-readable-p par))
-	    ()
-	  (insert-file-contents par)
-	  (setq path (file-name-directory par))
-	  ;; include children files
-	  (while (re-search-forward "^\<[ \t]*\\([^ \t\n]+\\).*$" nil t)
-	    (setq chr (expand-file-name (mew-match 1) path))
-	    (delete-region (match-beginning 0) (match-end 0))
-	    (if (and (file-readable-p chr) (not (member chr files)))
-		(progn
-		  (insert-file-contents chr)
-		  (setq files (cons chr files)))))
-	  (goto-char (point-max))))
-      ;; remove commets
-      (goto-char (point-min))
-      (while (re-search-forward "[;#].*$" nil t)
-	(delete-region (match-beginning 0) (match-end 0)))
-      ;; concat continuation lines
-      (goto-char (point-min))
-      (while (re-search-forward "\\\\\n" nil t)
-	(delete-region (match-beginning 0) (match-end 0)))
-      ;; concat separated lines by comma
-      (goto-char (point-min))
-      (while (re-search-forward ",$" nil t)
-	(end-of-line)
-	(forward-char 1)
-	(delete-backward-char 1))
-      ;; remove optional white spaces
-      (goto-char (point-min))
-      (while (re-search-forward "[ \t]+" nil t)
-	(replace-match " "))
-      ;;
-      (goto-char (point-min))
-      (while (re-search-forward "^ ?\\([^ \n:]+\\) ?: ?\\(.*\\)$" nil t)
-	(setq alias (mew-match 1))
-	(setq expn (mew-match 2))
-	;; append for first assoc comes first
-	(setq alist (cons (cons alias expn) alist)))
-      ;; load from mew-refile-from-alist
-      (setq alist (nreverse alist))
-      (mew-refile-alist-append-alias alist)))) ;; return value
+  (setq mew-config-cases nil))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -339,16 +339,18 @@ Currently, only \"user: user@domain\" syntax is supported."
 	     (file-readable-p file))
 	(insert-file-contents file))
        (t
-	(mapcar (lambda (x) (insert x "\n"))
+	(mapcar (function (lambda (x) (insert x "\n")))
 		(nconc (funcall mew-folder-list-function "+")
 		       (funcall mew-folder-list-function "=")))
+	(if mew-use-imap
+	    (mew-folder-mail-to-imap))
 	(if mew-use-folders-file-p
-	    (write-region (point-min) (point-max) file nil 'no-msg)))
-       )
+	    (write-region (point-min) (point-max) file nil 'no-msg))))
       (goto-char (point-min))
       (while (not (eobp))
 	(setq start (point))
-	(if (not (looking-at "[+=]"))
+	(if (not (or (looking-at "[+=]")
+		     (and mew-use-imap (looking-at "%"))))
 	    (forward-line)
 	  (forward-line)
 	  (setq folder (mew-buffer-substring start (1- (point))))
@@ -365,6 +367,12 @@ Currently, only \"user: user@domain\" syntax is supported."
 	    (setq folders (cons folder folders)))))
       (sort (nreverse folders) (function mew-string<)))))
 
+(defun mew-folder-mail-to-imap ()
+  (goto-char (point-min))
+  (if mew-imap-account
+      (while (re-search-forward "^+@[^#]+#[^/]+/\\(.*\\)$" nil t)
+	(replace-match (concat "%" (mew-match 1))))))
+
 (defmacro mew-folder-make-alist (list)
   (` (mapcar (function mew-folder-pair) (, list))))
 
@@ -395,29 +403,23 @@ Currently, only \"user: user@domain\" syntax is supported."
 	(not ret)
       ret)))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
-;;; Petnames
-;;;
+(defun mew-inbox-folder ()
+  (cond
+   ((string= mew-config-imget mew-config-default);; this may be lengthy
+    mew-inbox-folder)
+   ((assoc mew-config-imget mew-config-case-inbox)
+    (nth 1 (assoc mew-config-imget mew-config-case-inbox)))
+   (t 
+    mew-inbox-folder)))
 
-(defvar mew-petname-alist nil)
-
-(defun mew-petname-setup ()
-  (if mew-use-petname
-      (setq mew-petname-alist (mew-petname-make-alist))))
-
-(defun mew-petname-make-alist ()
-  (if (and mew-petname-file (file-readable-p mew-petname-file))
-      (save-excursion
-	(let (ret)
-	  (mew-set-buffer-tmp)
-	  (insert-file-contents mew-petname-file)
-	  (goto-char (point-min))
-	  (while (not (eobp))
-	    (if (looking-at "^\\([^ \t]+\\)[ \t]+\"\\(.*\\)\"$")
-		(setq ret (cons (cons (mew-match 1) (mew-match 2)) ret)))
-	    (forward-line))
-	  ret))))
+(defun mew-inbox-folders ()
+  (if (null mew-config-case-inbox)
+      (list mew-inbox-folder)
+    (let ((inboxes (mapcar (function (lambda (x) (nth 1 x)))
+			   mew-config-case-inbox)))
+      (if (member mew-inbox-folder inboxes)
+	  inboxes
+	(cons mew-inbox-folder inboxes)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -456,10 +458,9 @@ Currently, only \"user: user@domain\" syntax is supported."
 	       'window-id
 	       (frame-parameters (window-frame (selected-window)))))
 	 (sort
-	  (mapcar (lambda (frame)
-		    (cdr (assq
-			  'window-id
-			  (frame-parameters frame))))
+	  (mapcar (function (lambda (frame)
+			      (cdr (assq 'window-id
+					 (frame-parameters frame)))))
 		  (frame-list))
 	  (function string<)))))
     mew-buffer-message))
@@ -472,8 +473,6 @@ Currently, only \"user: user@domain\" syntax is supported."
 ;;; action : summary, message, draft or list
 ;;; list for action (1 0)  for Summary only
 ;;; list for action (3 10) for Summary and Message
-  (if (equal action 'summary)
-      (mew-current-set 'message nil))
   (let* ((windows
 	  (if (listp action) 
 	      action
@@ -501,7 +500,7 @@ Currently, only \"user: user@domain\" syntax is supported."
     (if (not (zerop (nth 0 windows)))
 	(setq sum-height (max window-min-height
 			     (/ (* height (nth 0 windows)) winsum))))
-    (if (and (equal action 'summary) (equal (% sum-height 2) 1)) 
+    (if (and (equal action 'message) (equal (% sum-height 2) 1)) 
 	(setq sum-height (1+ sum-height)))
     (if (not (zerop (nth 1 windows)))
 	(setq msg-height (max window-min-height
@@ -523,7 +522,7 @@ Currently, only \"user: user@domain\" syntax is supported."
 (defvar mew-buffers nil)
 
 (defun mew-buffers-init ()
-  (setq mew-buffers (list mew-inbox-folder))) ;; for quiting
+  (setq mew-buffers (mew-inbox-folders))) ;; for quiting
 
 (defun mew-buffers-setup (folder)
   (if (not (mew-folder-member folder mew-buffers))
@@ -575,6 +574,10 @@ It is typically called by kill-emacs-hook."
 ;;; Teer down
 ;;;
 
+(defun mew-bury-buffer (&optional buf)
+  (bury-buffer buf)
+  (delete-windows-on buf t))
+
 (defun mew-kill-buffer (&optional buf)
   "Erase the current mode(buffer)."
   (interactive)
@@ -582,28 +585,47 @@ It is typically called by kill-emacs-hook."
 	 (folder (if (bufferp buf) (buffer-name buf) buf)))
     (if (eq major-mode 'mew-virtual-mode)
 	(mew-folder-delete folder))
+    (if (and (eq major-mode 'mew-summary-mode)
+	     (mew-folder-remotep folder))
+	(mew-remote-folder-cache-delete folder))
     (if (get-buffer buf) (kill-buffer buf))))
 
 (defun mew-buffer-message-clean-up (func)
   (if window-system
-      (let ((x (buffer-list))
-	    (regexp (concat "^" (regexp-quote mew-buffer-message))))
-	(while x
-	  (if (and (buffer-name (car x))
-		   (string-match regexp (buffer-name (car x))))
-	      (funcall func (car x)))
-	  (setq x (cdr x))))
+      (let ((bl (buffer-list))
+	    (regexp (concat "^" (regexp-quote mew-buffer-message)))
+	    b bn)
+	(while bl
+	  (setq b (car bl))
+	  (setq bl (cdr bl))
+	  (if (and (setq bn (buffer-name b))
+		   (string-match regexp bn))
+	      (funcall func b))))
     (funcall func (mew-buffer-message))))
 
+(defun mew-buffer-draft-clean-up ()
+  (let ((bl (buffer-list))
+	(regexp (concat "^" (regexp-quote mew-draft-folder)))
+	b bn)
+    (while bl
+      (setq b (car bl))
+      (setq bl (cdr bl))
+      (if (and (setq bn (buffer-name b))
+	       (string-match regexp bn)
+	       (equal 'mew-draft-mode
+		      (save-excursion
+			(set-buffer b)
+			major-mode)))
+	  (mew-kill-buffer b)))))
+
 (defun mew-summary-suspend ()
   "Suspend Mew then switch to another buffer. All buffers of 
-Mew retain, so you can resume with buffer operations."
+Mew remain, so you can resume with buffer operations."
   (interactive)
-  (mew-window-pop)
-  ;; bury buffers
-  (mew-buffer-message-clean-up (function bury-buffer))
+  (mew-buffer-message-clean-up (function mew-bury-buffer))
   (mew-buffers-bury)
-  (set-buffer (car (buffer-list)))
+  (mew-window-pop)
+  (redraw-frame (window-frame (selected-window))) ;; update toolbar
   (run-hooks 'mew-suspend-hook))
 
 (defun mew-summary-quit ()
@@ -611,26 +633,30 @@ Mew retain, so you can resume with buffer operations."
   (interactive)
   (if (not (y-or-n-p "Quit Mew? "))
       ()
-    (mew-window-pop)
     ;; killing buffers
-    (mew-buffer-message-clean-up (function mew-kill-buffer))
+    (mew-buffer-message-clean-up (function mew-kill-buffer)) ;; Message mode
+    (mew-buffer-draft-clean-up) ;; Draft mode
     (mew-cache-clean-up)
-    (mew-kill-buffer mew-buffer-hello)
-    (mew-kill-buffer mew-buffer-mime)
-    (mew-kill-buffer mew-buffer-tmp)
+    (mapcar (function mew-kill-buffer) mew-buffer-list) ;; other buffers
     ;;
     (mew-mark-clean-up)
-    (mew-buffers-clean-up)
+    (mew-buffers-clean-up) ;; Summary mode and Virtual mode
     (mew-temp-dir-clean-up)
     ;;
     (run-hooks 'mew-quit-hook)
     ;;
     ;; lastly, clean up variables
     ;;
-    (mew-window-clean-up)
     (mew-folder-clean-up)
     (mew-current-clean-up)
-    (mew-config-clean-up)
+    (mew-addrbook-clean-up)
+    (mew-passwd-clean-up)
+    ;;
+    (mew-window-pop)
+    (mew-window-clean-up)
+    (redraw-frame (window-frame (selected-window))) ;; update toolbar
+    ;;
+    (mew-config-clean-up) ;; MUST be last
     ;; flush minibuffer
     (message "")))
 
@@ -638,6 +664,7 @@ Mew retain, so you can resume with buffer operations."
 ;;;
 ;;; Load Mew libraries
 ;;;
+(require 'mew-addrbook)
 (require 'mew-complete)
 (require 'mew-minibuf)
 (require 'mew-cache)
@@ -665,7 +692,7 @@ Mew retain, so you can resume with buffer operations."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1994, 1995, 1996, 1997, 1998 Mew developing team.
+;; Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
